# coding: utf-8


"""
    python-creole
    ~~~~~~~~~~~~~
   
    created by Jens Diemer

    :copyleft: 2009-2011 by the python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

BLOCK_TAGS = (
    "address",
    "blockquote",
    "center",
    "dir",
    "div",
    "dl",
    "fieldset",
    "form",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "hr",
    "ins",
    "isindex",
    "men",
    "noframes",
    "noscript",
    "ul",
    "ol",
    "li",
    "table",
    "th",
    "tr",
    "td",
    "p",
    "pre",
    "br",
)
IGNORE_TAGS = ("tbody",)



from django.contrib import admin
from .models import CMSPageTypes
from .models import CMSContents
from .models import CMSEntries
from .models import CMSMarkUps
from .models import CMSTemplates
from .models import CMSPaths


@admin.register(CMSPaths)
class CMSPathsAdmin(admin.ModelAdmin):
    list_display = ["id", "path", "parent"]


@admin.register(CMSContents)
class CMSContents(admin.ModelAdmin):
    list_display = ["id", "title", "timestamp", "markup", "page"]
    list_filter = ["title", "markup"]


def make_published(modeladmin, request, queryset):
    queryset.update(published=True)


make_published.short_description = "Publish Selected CMSEntries"


@admin.register(CMSEntries)
class CMSEntries(admin.ModelAdmin):
    list_display = [
        "id",
        "title",
        "slug",
        "path",
        "page_type",
        "published",
        "frontpage",
    ]
    list_filter = ["page_type", "template"]
    search_fields = ("title",)
    actions = [make_published]


@admin.register(CMSMarkUps)
class CMSMarkUps(admin.ModelAdmin):
    pass


@admin.register(CMSTemplates)
class CMSTemplates(admin.ModelAdmin):
    pass


@admin.register(CMSPageTypes)
class CMSPageTypesAdmin(admin.ModelAdmin):
    list_filter = ("page_type", "text", "view_class")
    list_display = ["id", "page_type", "text", "view_class", "view_template"]



#!/usr/bin/env python
# coding: utf-8

"""
    python-creole exceptions
    ~~~~~~~~~~~~~~~~~~~~~~~~
    
    :copyleft: 2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals


class DocutilsImportError(ImportError):
    pass



#!/usr/bin/env python
# coding: utf-8

"""
    python-creole
    ~~~~~~~~~~~~~


    :copyleft: 2008-2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import re
import sys
import warnings

from mycms.creole.html_parser.config import BLOCK_TAGS, IGNORE_TAGS
from mycms.creole.html_tools.strip_html import strip_html
from mycms.creole.py3compat import TEXT_TYPE, BINARY_TYPE
from mycms.creole.shared.document_tree import DocNode, DebugList
from mycms.creole.shared.html_parser import HTMLParser

# ------------------------------------------------------------------------------

block_re = re.compile(
    r"""
    ^<pre> \s* $
    (?P<pre_block>
        (\n|.)*?
    )
    ^</pre> \s* $
    [\s\n]*
""",
    re.VERBOSE | re.UNICODE | re.MULTILINE,
)

inline_re = re.compile(
    r"""
    <pre>
    (?P<pre_inline>
        (\n|.)*?
    )
    </pre>
""",
    re.VERBOSE | re.UNICODE,
)


headline_tag_re = re.compile(r"h(\d)", re.UNICODE)

# ------------------------------------------------------------------------------


class HtmlParser(HTMLParser):
    """
    parse html code and create a document tree.
    
    >>> p = HtmlParser()
    >>> p.feed("<p>html <strong>code</strong></p>")
    <DocNode document: None>
    >>> p.debug()
    ________________________________________________________________________________
      document tree:
    ================================================================================
    p
        data: 'html '
        strong
            data: 'code'
    ********************************************************************************
    
    >>> p = HtmlParser()
    >>> p.feed("<p>html1 <script>var foo='<em>BAR</em>';</script> html2</p>")
    <DocNode document: None>
    >>> p.debug()
    ________________________________________________________________________________
      document tree:
    ================================================================================
    p
        data: 'html1 '
        script
            data: "var foo='<em>BAR"
            data: '</em>'
            data: "';"
        data: ' html2'
    ********************************************************************************
    """

    # placeholder html tag for pre cutout areas:
    _block_placeholder = "blockdata"
    _inline_placeholder = "inlinedata"

    def __init__(self, debug=False):
        HTMLParser.__init__(self)

        self.debugging = debug
        if self.debugging:
            warnings.warn(message="Html2Creole debug is on! warn every data append.")
            self.result = DebugList(self)
        else:
            self.result = []

        self.blockdata = []

        self.root = DocNode("document", None)
        self.cur = self.root

        self.__list_level = 0

    def _pre_cut(self, data, type, placeholder):
        if self.debugging:
            print("append blockdata: %r" % data)
        assert isinstance(data, TEXT_TYPE), "blockdata is not unicode"
        self.blockdata.append(data)
        id = len(self.blockdata) - 1
        return '<%s type="%s" id="%s" />' % (placeholder, type, id)

    def _pre_pre_inline_cut(self, groups):
        return self._pre_cut(groups["pre_inline"], "pre", self._inline_placeholder)

    def _pre_pre_block_cut(self, groups):
        return self._pre_cut(groups["pre_block"], "pre", self._block_placeholder)

    def _pre_pass_block_cut(self, groups):
        content = groups["pass_block"].strip()
        return self._pre_cut(content, "pass", self._block_placeholder)

    _pre_pass_block_start_cut = _pre_pass_block_cut

    def _pre_cut_out(self, match):
        groups = match.groupdict()
        for name, text in groups.items():
            if text is not None:
                if self.debugging:
                    print("%15s: %r (%r)" % (name, text, match.group(0)))
                method = getattr(self, "_pre_%s_cut" % name)
                return method(groups)

    #        data = match.group("data")

    def feed(self, raw_data):
        assert isinstance(raw_data, TEXT_TYPE), "feed data must be unicode!"
        data = raw_data.strip()

        # cut out <pre> and <tt> areas block tag areas
        data = block_re.sub(self._pre_cut_out, data)
        data = inline_re.sub(self._pre_cut_out, data)

        # Delete whitespace from html code
        data = strip_html(data)

        if self.debugging:
            print("_" * 79)
            print("raw data:")
            print(repr(raw_data))
            print(" -" * 40)
            print("cleaned data:")
            print(data)
            print("-" * 79)
        #            print(clean_data.replace(">", ">\n"))
        #            print("-"*79)

        HTMLParser.feed(self, data)

        return self.root

    # -------------------------------------------------------------------------

    def _upto(self, node, kinds):
        """
        Look up the tree to the first occurence
        of one of the listed kinds of nodes or root.
        Start at the node node.
        """
        while node is not None and node.parent is not None:
            node = node.parent
            if node.kind in kinds:
                break

        return node

    def _go_up(self):
        kinds = list(BLOCK_TAGS) + ["document"]
        self.cur = self._upto(self.cur, kinds)
        self.debug_msg("go up to", self.cur)

    # -------------------------------------------------------------------------

    def handle_starttag(self, tag, attrs):
        self.debug_msg("starttag", "%r atts: %s" % (tag, attrs))

        if tag in IGNORE_TAGS:
            return

        headline = headline_tag_re.match(tag)
        if headline:
            self.cur = DocNode("headline", self.cur, level=int(headline.group(1)))
            return

        if tag in ("li", "ul", "ol"):
            if tag in ("ul", "ol"):
                self.__list_level += 1
            self.cur = DocNode(tag, self.cur, None, attrs, level=self.__list_level)
        elif tag in ("img", "br"):
            # Work-a-round if img or br  tag is not marked as startendtag:
            # wrong: <img src="/image.jpg"> doesn't work if </img> not exist
            # right: <img src="/image.jpg" />
            DocNode(tag, self.cur, None, attrs)
        else:
            self.cur = DocNode(tag, self.cur, None, attrs)

    def handle_data(self, data):
        self.debug_msg("data", "%r" % data)
        if isinstance(data, BINARY_TYPE):
            data = unicode(data)
        DocNode("data", self.cur, content=data)

    def handle_charref(self, name):
        self.debug_msg("charref", "%r" % name)
        DocNode("charref", self.cur, content=name)

    def handle_entityref(self, name):
        self.debug_msg("entityref", "%r" % name)
        DocNode("entityref", self.cur, content=name)

    def handle_startendtag(self, tag, attrs):
        self.debug_msg("startendtag", "%r atts: %s" % (tag, attrs))
        attr_dict = dict(attrs)
        if tag in (self._block_placeholder, self._inline_placeholder):
            id = int(attr_dict["id"])
            #            block_type = attr_dict["type"]
            DocNode(
                "%s_%s" % (tag, attr_dict["type"]),
                self.cur,
                content=self.blockdata[id],
                #                attrs = attr_dict
            )
        else:
            DocNode(tag, self.cur, None, attrs)

    def handle_endtag(self, tag):
        if tag in IGNORE_TAGS:
            return

        self.debug_msg("endtag", "%r" % tag)

        if tag == "br":  # handled in starttag
            return

        self.debug_msg("starttag", "%r" % self.get_starttag_text())

        if tag in ("ul", "ol"):
            self.__list_level -= 1

        if tag in BLOCK_TAGS or self.cur is None:
            self._go_up()
        else:
            self.cur = self.cur.parent

    # -------------------------------------------------------------------------

    def debug_msg(self, method, txt):
        if not self.debugging:
            return
        print("%-8s %8s: %s" % (self.getpos(), method, txt))

    def debug(self, start_node=None):
        """
        Display the current document tree
        """
        print("_" * 80)

        if start_node == None:
            start_node = self.root
            print("  document tree:")
        else:
            print("  tree from %s:" % start_node)

        print("=" * 80)

        def emit(node, ident=0):
            for child in node.children:
                txt = "%s%s" % (" " * ident, child.kind)

                if child.content:
                    txt += ": %r" % child.content

                if child.attrs:
                    txt += " - attrs: %r" % child.attrs

                if child.level != None:
                    txt += " - level: %r" % child.level

                print(txt)
                emit(child, ident + 4)

        emit(start_node)
        print("*" * 80)


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())

#    p = HtmlParser(debug=True)
#    p.feed("""\
# <p><span>in span</span><br />
# <code>in code</code></p>
# """)
#    p.debug()



class MarkupTable(object):
    """
    Container for holding table data and render the data in creole markup.
    Format every cell width to the same col width.
    """

    def __init__(self, head_prefix="= ", auto_width=True, debug_msg=None):
        self.head_prefix = head_prefix
        self.auto_width = auto_width

        if debug_msg is None:
            self.debug_msg = self._non_debug
        else:
            self.debug_msg = debug_msg

        self.rows = []
        self.row_index = None
        self.has_header = False

    def _non_debug(self, *args):
        pass

    def add_tr(self):
        self.debug_msg("Table.add_tr", "")
        self.rows.append([])
        self.row_index = len(self.rows) - 1

    def add_th(self, text):
        self.has_header = True
        self.add_td(self.head_prefix + text)

    def add_td(self, text):
        if self.row_index == None:
            self.add_tr()

        self.debug_msg("Table.add_td", text)
        self.rows[self.row_index].append(text)

    def _get_preformat_info(self):
        cells = []
        for row in self.rows:
            line_cells = []
            for cell in row:
                cell = cell.strip()
                if cell != "":
                    if self.head_prefix and cell.startswith(self.head_prefix):
                        cell += " "  # Headline
                    else:
                        cell = " %s " % cell  # normal cell
                line_cells.append(cell)
            cells.append(line_cells)

        # Build a list of max len for every column
        widths = [max(map(len, col)) for col in zip(*cells)]

        return cells, widths

    def get_table_markup(self):
        """ return the table data in creole/textile markup. """
        if not self.auto_width:
            lines = []
            for row in self.rows:
                lines.append("|" + "|".join([cell for cell in row]) + "|")
        else:
            # preformat every table cell
            cells, widths = self._get_preformat_info()

            # Join every line with ljust
            lines = []
            for row in cells:
                cells = [cell.ljust(width) for cell, width in zip(row, widths)]
                lines.append("|" + "|".join(cells) + "|")

        result = "\n".join(lines)

        self.debug_msg("Table.get_table_markup", result)
        return result

    def get_rest_table(self):
        """ return the table data in ReSt markup. """
        # preformat every table cell
        cells, widths = self._get_preformat_info()

        separator_line = "+%s+" % "+".join(["-" * width for width in widths])
        headline_separator = "+%s+" % "+".join(["=" * width for width in widths])

        lines = []
        for no, row in enumerate(cells):
            if no == 1 and self.has_header:
                lines.append(headline_separator)
            else:
                lines.append(separator_line)

            # Join every line with ljust
            cells = [cell.ljust(width) for cell, width in zip(row, widths)]
            lines.append("|" + "|".join(cells) + "|")

        lines.append(separator_line)

        return "\n".join(lines)


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())



# coding: utf-8


"""
    Creole Rules for parser
    ~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2008-2013 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import re


class InlineRules(object):
    """
    All inline rules
    """

    proto = r"http|https|ftp|nntp|news|mailto|telnet|file|irc"
    # New regex for finding uris, requires uri to free stand within whitespace or lineends.
    url = (
        r"""(?P<url>
            (^ | (?<=\s))
            (?P<escaped_url>~)?
            (?P<url_target> (?P<url_proto> %s )://[^$\s]+ )
        )"""
        % proto
    )
    # Original uri matching regex inherited from MoinMoin code.
    # url = r'''(?P<url>
    # (^ | (?<=\s | [.,:;!?()/=]))
    # (?P<escaped_url>~)?
    # (?P<url_target> (?P<url_proto> %s ):\S+? )
    # ($ | (?=\s | [,.:;!?()] (\s | $)))
    # )''' % proto
    link = r"""(?P<link>
            \[\[
            (?P<link_target>.+?) \s*
            ([|] \s* (?P<link_text>.+?) \s*)?
            ]]
        )"""

    #    link = r'''(?P<link1>
    #            \[\[
    #            (?P<link_target1>.+?)\|(?P<link_text1>.+?)
    #            ]]
    #        )|(?P<link2>
    #            \[\[
    #            (?P<link_target2> (%s)://[^ ]+) \s* (?P<link_text2>.+?)
    #            ]]
    #        )|
    #            \[\[(?P<internal_link>.+)\]\]
    #        ''' % proto

    # image tag
    image = r"""(?P<image>
            {{
            (?P<image_target>.+?) \s*
            (\| \s* (?P<image_text>.+?) \s*)?
            }}
        )(?i)"""
    # --------------------------------------------------------------------------

    # a macro like: <<macro>>text<</macro>>
    macro_inline = r"""
        (?P<macro_inline>
        << \s* (?P<macro_inline_start>\w+) \s* (?P<macro_inline_args>.*?) \s* >>
        (?P<macro_inline_text>(.|\n)*?)
        <</ \s* (?P=macro_inline_start) \s* >>
        )
    """
    # A single macro tag, like <<macro-a foo="bar">> or <<macro />>
    macro_tag = r"""(?P<macro_tag>
            <<(?P<macro_tag_name> \w+) (?P<macro_tag_args>.*?) \s* /*>>
        )"""

    pre_inline = r"(?P<pre_inline> {{{ (?P<pre_inline_text>.*?) }}} )"

    # Basic text typefaces:

    emphasis = r"(?P<emphasis>(?<!:)// (?P<emphasis_text>.+?) (?<!:)// )"
    # there must be no : in front of the // avoids italic rendering
    # in urls with unknown protocols

    strong = r"(?P<strong>\*\* (?P<strong_text>.+?) \*\* )"

    # Creole 1.0 optional:
    monospace = r"(?P<monospace> \#\# (?P<monospace_text>.+?) \#\# )"
    superscript = r"(?P<superscript> \^\^ (?P<superscript_text>.+?) \^\^ )"
    subscript = r"(?P<subscript> ,, (?P<subscript_text>.+?) ,, )"
    underline = r"(?P<underline> __ (?P<underline_text>.+?) __ )"
    delete = r"(?P<delete> ~~ (?P<delete_text>.+?) ~~ )"

    # own additions:
    small = r"(?P<small>-- (?P<small_text>.+?) -- )"

    linebreak = r"(?P<linebreak> \\\\ )"
    escape = r"(?P<escape> ~ (?P<escaped_char>\S) )"
    char = r"(?P<char> . )"


class BlockRules(object):
    """
    All used block rules.
    """

    #    macro_block = r'''(?P<macro_block>
    #            \s* << (?P<macro_block_start>\w+) \s* (?P<macro_block_args>.*?) >>
    #            (?P<macro_block_text>(.|\n)+?)
    #            <</(?P=macro_block_start)>> \s*
    #        )'''
    #    macro_block = r'''(?P<macro_block>
    #            <<(?P<macro_block_start>.*?)>>
    #            (?P<macro_block_text>.*?)
    #            <</.*?>>
    #        )'''

    macro_block = r"""
        (?P<macro_block>
        << \s* (?P<macro_block_start>\w+) \s* (?P<macro_block_args>.*?) \s* >>
        (?P<macro_block_text>(.|\n)*?)
        <</ \s* (?P=macro_block_start) \s* >>
        )
    """

    line = r"""(?P<line> ^\s*$ )"""  # empty line that separates paragraphs

    head = r"""(?P<head>
        ^
        (?P<head_head>=+) \s*
        (?P<head_text> .*? )
        (=|\s)*?$
    )"""
    separator = r"(?P<separator> ^ \s* ---- \s* $ )"  # horizontal line

    pre_block = r"""(?P<pre_block>
            ^{{{ \s* $
            (?P<pre_block_text>
                ([\#]!(?P<pre_block_kind>\w*?)(\s+.*)?$)?
                (.|\n)+?
            )
            ^}}})
        """

    # Matches the whole list, separate items are parsed later. The
    # list *must* start with a single bullet.
    list = r"""(?P<list>
        ^ [ \t]* ([*][^*\#]|[\#][^\#*]).* $
        ( \n[ \t]* [*\#]+.* $ )*
    )"""

    table = r"""^ \s*(?P<table>
            [|].*? \s*
            [|]?
        ) \s* $"""

    re_flags = re.VERBOSE | re.UNICODE | re.MULTILINE

    def __init__(self, blog_line_breaks=True):
        if blog_line_breaks:
            # use blog style line breaks (every line break would be converted into <br />)
            self.text = r"(?P<text> .+ ) (?P<break> (?<!\\)$\n(?!\s*$) )?"
        else:
            # use wiki style line breaks, seperate lines with one space
            self.text = r"(?P<space> (?<!\\)$\n(?!\s*$) )? (?P<text> .+ )"

        self.rules = (
            self.macro_block,
            self.line,
            self.head,
            self.separator,
            self.pre_block,
            self.list,
            self.table,
            self.text,
        )


class SpecialRules(object):
    """
    re rules witch not directly used as inline/block rules.
    """

    # Matches single list items:
    item = r"""^ \s* (?P<item>
        (?P<item_head> [\#*]+) \s*
        (?P<item_text> .*?)
    ) \s* $"""

    # For splitting table cells:
    cell = r"""
            \| \s*
            (
                (?P<head> [=][^|]+ ) |
                (?P<cell> (  %s | [^|])+ )
            ) \s*
        """ % "|".join(
        [
            InlineRules.link,
            InlineRules.macro_inline,
            InlineRules.macro_tag,
            InlineRules.image,
            InlineRules.pre_inline,
        ]
    )

    # For pre escaping, in creole 1.0 done with ~:
    pre_escape = r" ^(?P<indent>\s*) ~ (?P<rest> \}\}\} \s*) $"


INLINE_FLAGS = re.VERBOSE | re.UNICODE
INLINE_RULES = (
    InlineRules.link,
    InlineRules.url,
    InlineRules.macro_inline,
    InlineRules.macro_tag,
    InlineRules.pre_inline,
    InlineRules.image,
    InlineRules.strong,
    InlineRules.emphasis,
    InlineRules.monospace,
    InlineRules.underline,
    InlineRules.superscript,
    InlineRules.subscript,
    InlineRules.small,
    InlineRules.delete,
    InlineRules.linebreak,
    InlineRules.escape,
    InlineRules.char,
)


def _verify_rules(rules, flags):
    """
    Simple verify the rules -> try to compile it ;)
    
    >>> _verify_rules(INLINE_RULES, INLINE_FLAGS)
    Rule test ok.
    
    >>> block_rules = BlockRules()   
    >>> _verify_rules(block_rules.rules, block_rules.re_flags)
    Rule test ok.
    """
    # Test with re.compile
    rule_list = []
    for rule in rules:
        try:
            #            print(rule)
            re.compile(rule, flags)

            # Try to merge the rules. e.g. Check if group named double used.
            rule_list.append(rule)
            re.compile("|".join(rule_list), flags)
        except Exception as err:
            print(" *** Error with rule:")
            print(rule)
            print(" -" * 39)
            raise
    print("Rule test ok.")


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())

    print("-" * 80)



# coding: utf-8

"""
    utils for distutils setup
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    Get README.creole as ReStructuredText on-the-fly for setup.long_description

    More information:
        https://code.google.com/p/python-creole/wiki/UseInSetup

    usage in setup.py e.g.:
    ---------------------------------------------------------------------------
    #!/usr/bin/env python
    # coding: utf-8

    import os
    import sys
    from setuptools import setup, find_packages

    PACKAGE_ROOT = os.path.dirname(os.path.abspath(__file__))

    try:
        from mycms.creole.setup_utils import get_long_description
    except ImportError:
        if "register" in sys.argv or "sdist" in sys.argv or "--long-description" in sys.argv:
            etype, evalue, etb = sys.exc_info()
            evalue = etype("%s - Please install python-creole >= v0.8 -  e.g.: pip install python-creole" % evalue)
            raise etype, evalue, etb
        long_description = None
    else:
        long_description = get_long_description(PACKAGE_ROOT)

    setup(
        ...
        long_description = long_description,
        ...
    )
    ---------------------------------------------------------------------------

    :copyleft: 2011-2014 by the python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import codecs
import os
import sys
import warnings

from mycms.creole import creole2html, html2rest
from mycms.creole.shared.unknown_tags import (
    raise_unknown_node,
    transparent_unknown_nodes,
)
from mycms.creole.py3compat import PY3


RAISE_ERRORS_ARGS = (
    "check",
    "register",
    "sdist",
    "bdist",
    "upload",
    "--long-description",
    "--restructuredtext",
)


def should_raise_errors():
    """
    Raise only errors, if one of RAISE_ERRORS_ARGS is in sys.argv
    or if no argument presents.
    """
    if len(sys.argv) == 1:
        return True

    for arg in RAISE_ERRORS_ARGS:
        if arg in sys.argv:
            return True
    return False


def get_long_description(package_root, filename="README.creole", raise_errors=None):
    """ read README file and convert it on-the-fly to ReStructuredText. """
    if raise_errors is None:
        raise_errors = should_raise_errors()

    if raise_errors:
        sys.stderr.write(
            "Test creole2rest and raise an error, if rendering failed...\n"
        )
        # raise a error if a unknown node found
        unknown_emit = raise_unknown_node
    else:
        # ignore unknown nodes
        unknown_emit = transparent_unknown_nodes

    filepath = os.path.join(package_root, filename)
    long_description_origin = ""
    try:
        # Read creole README
        f = codecs.open(filepath, "r", encoding="utf-8")
        long_description_origin = f.read().strip()
        f.close()

        # convert creole into html
        long_description_html = creole2html(long_description_origin)

        # convert html to ReSt
        long_description_rest_unicode = html2rest(
            long_description_html, emitter_kwargs={"unknown_emit": unknown_emit}
        )
        if PY3:
            long_description_rest = long_description_rest_unicode
        else:
            long_description_rest = long_description_rest_unicode.encode("utf-8")
    except Exception:
        if raise_errors:
            raise
        # Don't raise the error e.g. in ./setup install process
        evalue = sys.exc_info()[1]
        long_description_rest = "[Error: %s]\n%s" % (evalue, long_description_origin)
    else:
        if raise_errors:
            # Test created ReSt code like PyPi does it.
            from mycms.creole.rest2html.pypi_rest2html import pypi_rest2html

            try:
                pypi_rest2html(long_description_rest_unicode)
            except SystemExit as e:
                msg = (
                    "Error creole2rest self test failed: rest2html() exist with status code: %s\n"
                    % e.args[0]
                )
                sys.stderr.write(msg)
                sys.exit(msg)
            except Exception as e:
                sys.exit("ReSt2html error: %s" % e)
            else:
                if "check" in sys.argv:
                    print("Generating creole to ReSt to html, ok.")

    return long_description_rest


def _get_long_description(*args, **kwargs):
    msg = "'GetLongDescription' is deprecated, use 'get_long_description' instead."
    if should_raise_errors():
        raise DeprecationWarning(msg)
    else:
        warnings.warn(msg, DeprecationWarning)
    return get_long_description(*args, **kwargs)


GetLongDescription = _get_long_description  # for backward-compatibility


if __name__ == "__main__":
    package_root = os.path.abspath("../")
    long_description = get_long_description(package_root)
    print(long_description)



import logging

from django import template
from django.core.exceptions import ObjectDoesNotExist
from django.template import Template, Context
from django.template.loader import get_template


# from mycms.models import Paths, Pages
# from mycms.pageview.base import get_pageview

from mycms.models import CMSPaths
from mycms.models import CMSEntries

register = template.Library()

logger = logging.getLogger("mycms.templatetags")


@register.inclusion_tag("mycms/templatetags/menus/dropdown_menu.html")
def dropdown_menu(path):
    """
    This is used to provide a dropdown nested menu for a certain category. 
    
    This is used in the front page of mycms to provide a list of main site sections
    for example.
    """

    try:
        parent = CMSEntries.objects.get(path__path=path)
    except ObjectDoesNotExist as e:
        msg = "No CMSEntries to produce dropdown_menu for path: {}".format(path)
        logger.fatal(msg)
        parent = []
    return {"parent": parent}


@register.inclusion_tag("mycms/templatetags/full_menu.html")
def full_menu():
    """
    Used to get a full category tree starting from /cms/. 
    """

    try:
        parent = CMSEntries.objects.get(path__path="/")
    except ObjectDoesNotExist as e:
        msg = (
            "No CMSEntries to produce full_menu from /. Perhaps / does not yet exist!!"
        )
        logger.fatal(msg)
        parent = None
    return {"parent": parent}


@register.inclusion_tag("mycms/templatetags/mini_menu.html")
def mini_menu():
    try:
        parent = CMSEntries.objects.get(path__path="/")
    except ObjectDoesNotExist as e:
        msg = (
            "No CMSEntries to produce full_menu from /. Perhaps / does not yet exist!!"
        )
        logger.fatal(msg)
        parent = None
    return {"parent": parent}



"""
The module mycms.api is a DRF based api which allows REST calls into the CMS contents stored
in the mycms database. The REST API documentation and interface can be reached at /cms/api/v2/docs/ of a mycms instance.

The web interface to the api exposes all the possible operations that we can do to the MyCMS database and is the
best way for understanding and testing the various api calls.

The API exposes the following endpoints:

* cmsauthtoken

* cmscontents

* cmsentries

* cmspages

* cmspaths



Tutorials
**********

Create a new CMSEntry using javascript
---------------------------------------

Introduction and Some Background:

Every mycms page defines a view_json object which contains information about the page. This can be seen in the html page
for example:

.. code-block:: javascript

    var view_json = {"id": 1, "title": "Yet Another CMS.", "path": 1, "slug": "yet-another-cms", "page_type": 5, "template": null, "frontpage": false, "published": true, "lists_include": false, "page_number": 1, "created_by": 1, "logo_url": "/static/mycms/images/png/default.png", "path_str": "/", "content": [], "date_created_epoch": 1460237699000, "date_modified_epoch": 1460237699000}


In order to create a CMSEntry we use the endpoint /cms/api/v2/cmsentries/id/create_child/ where id is the parent of the page we want to create.

"""


from django.core.exceptions import ObjectDoesNotExist

# from rest_framework.renderers import JSONRenderer
# from rest_framework.parsers import JSONParser
from rest_framework import serializers

# from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework import viewsets
# from rest_framework import views as drf_views
from rest_framework import status
from rest_framework.views import APIView
from rest_framework.decorators import action

from rest_framework.authtoken.models import Token

from mycms.serializers import CMSPageTypesSerializer
from mycms.serializers import CMSContentsSerializer
from mycms.serializers import CMSEntrySerializer
# from mycms.serializers import CMSMarkUpSerializer
# from mycms.serializers import CMSTemplatesSerializer
from mycms.serializers import CMSPathsSerializer
# from mycms.serializers import CMSEntryExpandedSerializer
# from mycms.serializers import LoremIpsumSerializer
from mycms.serializers import CMSChildEntrySerializer

import mycms.serializers as mycmsserializers

from mycms.models import CMSContents
# from mycms.models import CMSMarkUps
# from mycms.models import CMSTemplates
from mycms.models import CMSPageTypes
from mycms.models import CMSEntries
from mycms.models import CMSPaths

from rest_framework.schemas import AutoSchema, ManualSchema
import coreapi
import coreschema


from django_filters.rest_framework import DjangoFilterBackend

__all__ = ["CMSContentsViewSet", "CMSFormatterContent", "CMSEntriesViewSet"]


class CMSContentsViewSet(viewsets.ModelViewSet):
    """
    A viewset that allows
    """

    permission_classes = (IsAuthenticated,)

    queryset = CMSContents.objects.all()
    serializer_class = CMSContentsSerializer

    @action(detail=True, methods=["get"])
    def html(self, request, pk=None):

        content_obj = CMSContents.objects.get(id=pk)
        data = {"html": content_obj.html}
        return Response(data, status=status.HTTP_200_OK)


class CMSFormatterContent(APIView):
    def get(self, request, **kwargs):

        content_id = kwargs.get("content_id")

        content_obj = CMSContents.objects.get(id=content_id)
        data = {"html": content_obj.html}
        data = {"html": "Use the new html action of CMSContents."}
        return Response(data, status=status.HTTP_200_OK)


class CMSEntriesViewSet(viewsets.ModelViewSet):
    permission_classes = (IsAuthenticated,)

    serializer_class = CMSEntrySerializer
    filter_backends = (DjangoFilterBackend,)
    filter_fields = ("page_type", "frontpage", "published")

    def get_queryset(self):

        queryset = CMSEntries.objects.all()

        parent_path_id = self.request.query_params.get("parent_path_id", None)

        if parent_path_id is not None:
            queryset = queryset.filter(path__parent=parent_path_id)
        return queryset

    @action(detail=True, methods=["get"])
    def get_categories(self, request, pk=None):
        parent_obj = CMSEntries.objects.get(id=pk)
        print(parent_obj)
        c = CMSEntries.objects.filter(
            path__parent=parent_obj.path, page_type__page_type="CATEGORY"
        )
        serializer = CMSEntrySerializer(c, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @action(detail=True,methods=["post"])
    def create_child(self, request, pk=None):
        """
        A utility function to create an article including path information.
        """
        print(request.data)
        fake_flag = request.GET.get("fake", False)
        request.POST._mutable = True
        title = request.data.get("title", None)
        slug = request.data.get("slug", None)

        if fake_flag:
            from faker import Faker
            import random
            from django.utils.text import slugify

            fake_factory = Faker()

            fake_title = " ".join(fake_factory.words(random.randint(3, 7)))

            if (title is None) or (len(title) == 0):
                request.data["title"] = fake_title.capitalize()
                request.data["slug"] = slugify(fake_title)

            request.data["published"] = True

        serializer = CMSChildEntrySerializer(data=request.data)

        if serializer.is_valid():
            print(serializer.data)
            vd = serializer.validated_data

            # The CMSChildEntrySerializer expects to get the pk of
            # the parent.
            child_obj = serializer.create(vd, pk)
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        else:
            print(serializer.errors)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @action(detail=True,methods=["get"])
    def toggle_published(self, request, *args, **kwargs):

        try:
            pk = kwargs.get("pk", None)

            cms_entry = CMSEntries.objects.get(pk=pk)
            cms_entry.toggle_published()
            cms_entry.save()
        except Exception as e:

            return Response(
                {"error": "{}".format(e)}, status=status.HTTP_400_BAD_REQUEST
            )

        return Response(
            {"published": cms_entry.published}, status=status.HTTP_202_ACCEPTED
        )

    @action(detail=True,methods=["get"])
    def toggle_frontpage(self, request, *args, **kwargs):

        try:
            pk = kwargs.get("pk", None)

            cms_entry = CMSEntries.objects.get(pk=pk)
            cms_entry.toggle_frontpage()
            cms_entry.save()
        except Exception as e:

            return Response(
                {"error": "{}".format(e)}, status=status.HTTP_400_BAD_REQUEST
            )

        return Response(
            {"frontpage": cms_entry.frontpage}, status=status.HTTP_202_ACCEPTED
        )


class CMSPageTypeViewSet(viewsets.ModelViewSet):

    permission_classes = (IsAuthenticated,)
    queryset = CMSPageTypes.objects.all()
    serializer_class = CMSPageTypesSerializer


class CMSPathsViewSet(viewsets.ModelViewSet):

    permission_classes = (IsAuthenticated,)
    queryset = CMSPaths.objects.all()
    serializer_class = CMSPathsSerializer


class CMSPagesViewSet(viewsets.ModelViewSet):

    permission_classes = (IsAuthenticated,)
    queryset = CMSEntries.objects.all()
    serializer_class = mycmsserializers.CMSPageSerializer


class CMSContentPreview(viewsets.GenericViewSet):
    """Implements API endpoint to preview a page."""

    # from rest_framework.schemas.inspectors import AuthoSchema
    schema = ManualSchema(
        fields=[
            coreapi.Field(
                "content",
                required=True,
                location="form",
                schema=coreschema.String(
                    description="The raw content that needs to be formatted."
                ),
            )
        ],
        description="Retrieves the Creole formatted content of what is posted.",
    )

    # def get(self, request):

    # if request.user.is_authenticated:

    # token =

    def retrieve(self, request, **kwargs):

        content = request.data.get("content", None)

        if content:
            from mycms.view_handlers.formatters import CreoleFormatter

            content = CreoleFormatter(content).html()
            return Response(data={"content": content}, status=status.HTTP_200_OK)

        else:
            return Response(
                data={"error": "No Content"}, status=status.HTTP_400_BAD_REQUEST
            )


class CMSAuthToken(viewsets.GenericViewSet):
    """Implements retrieving of Token."""

    # permission_classes = (IsAuthenticated,)

    # from rest_framework.schemas.inspectors import AuthoSchema
    schema = ManualSchema(
        fields=[
            coreapi.Field(
                "username",
                required=True,
                location="form",
                schema=coreschema.String(
                    description="username required to create or retrieve token"
                ),
            ),
            coreapi.Field(
                "password",
                required=True,
                location="form",
                schema=coreschema.String(
                    description="password required to create or retrieve token"
                ),
            ),
            coreapi.Field(
                "renew",
                required=False,
                location="query",
                schema=coreschema.Boolean(
                    description="set to true to retrieve a new token invalidating old one if it exists."
                ),
                description="password required to create or retrieve token",
            ),
        ],
        description="Gets or Creates a Token for the given user.",
    )

    # def get(self, request):

    # if request.user.is_authenticated:

    # token =

    def retrieve(self, request, **kwargs):

        username = request.data.get("username", None)
        password = request.data.get("password", None)
        renew = request.data.get("renew", False)
        """Returns token for logged in user."""

        if request.user.is_authenticated:
            if renew:
                try:
                    token = Token.objects.get(user=request.user)
                    token.delete()
                except ObjectDoesNotExist as e:
                    # Nothing to renew
                    pass

            token, created = Token.objects.get_or_create(user=request.user)
            return Response(data={"token": token.key}, status=status.HTTP_200_OK)

        else:
            return Response(
                data={"error": "Not Authorized"}, status=status.HTTP_401_UNAUTHORIZED
            )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-05-12 09:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mycms", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="CMSArchivesIndex",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("month", models.IntegerField(default=0)),
                ("year", models.IntegerField(default=1975)),
                ("entries", models.ManyToManyField(blank=True, to="mycms.CMSEntries")),
            ],
        ),
    ]



# coding: utf-8

"""
    HTMLParser for Python 2.x and 3.x
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    The HTMLParser has problems with the correct handling of <script>...</script>
    and <style>...</style> areas.
       
    It was fixed with v2.7.3 and 3.2.3, see:
        http://www.python.org/download/releases/2.7.3/
        http://www.python.org/download/releases/3.2.3/
    see also:
        http://bugs.python.org/issue670664#msg146770
        
    :copyleft: 2011-2012 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


try:
    import HTMLParser as OriginHTMLParser
except ImportError:
    from html import parser as OriginHTMLParser  # python 3


if hasattr(OriginHTMLParser, "cdata_elem"):
    # Current python version is patched -> use the original
    HTMLParser = OriginHTMLParser
else:
    # Current python version is not patched -> use own patched version
    from mycms.creole.shared.HTMLParsercompat import HTMLParser



from django.core.management.base import BaseCommand, CommandError
from mycms.models import CMSEntries


from django.core.exceptions import ObjectDoesNotExist
from django.core.exceptions import MultipleObjectsReturned

from mycms.view_handlers.page_types import singlepageview_pagetype_obj
from mycms.view_handlers.page_types import multipageview_pagetype_obj
from mycms.view_handlers.page_types import allarticles_pagetype_obj

from django.contrib.auth.management.commands import createsuperuser
from django.core.management import CommandError


class Command(createsuperuser.Command):
    help = "Crate a superuser, and allow password to be provided"

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--password",
            dest="password",
            default=None,
            help="Specifies the password for the superuser.",
        )

    def handle(self, *args, **options):
        password = options.get("password")
        username = options.get("username")
        database = options.get("database")

        if password and not username:
            raise CommandError("--username is required if specifying --password")

        super(Command, self).handle(*args, **options)

        if password:
            user = self.UserModel._default_manager.db_manager(database).get(
                username=username
            )
            user.set_password(password)
            user.save()



import unittest
import mycms
from . import tests_classes


from django.core.management.base import BaseCommand, CommandError
from mycms.models import CMSEntries

import shutil, tempfile
from os import path
import unittest

from django.core.exceptions import ObjectDoesNotExist
from django.core.exceptions import MultipleObjectsReturned

from mycms.view_handlers.page_types import singlepageview_pagetype_obj
from mycms.view_handlers.page_types import multipageview_pagetype_obj
from mycms.view_handlers.page_types import allarticles_pagetype_obj


class TestExample(unittest.TestCase):
    def setUp(self):
        # Create a temporary directory
        self.test_dir = tempfile.mkdtemp()

    def tearDown(self):
        # Remove the directory after the test
        shutil.rmtree(self.test_dir)

    def test_something(self):
        # Create a file in the temporary directory
        f = open(path.join(self.test_dir, "test.txt"), "w")
        # Write something to it
        f.write("The owls are not what they seem")
        # Reopen the file and check if what we read back is the same
        f = open(path.join(self.test_dir, "test.txt"))
        self.assertEqual(f.read(), "The owls are not what they seem")


import sys


class Command(BaseCommand):

    """
    Example on how to create and run a unittest from within the django 
    management commands. 
    """

    def add_arguments(self, parser):
        parser.add_argument("--test_classes", nargs="+", type=str)

    def handle(self, *args, **options):

        suite = unittest.TestSuite()
        test_class_names = options.get("test_classes", None)

        if test_class_names:

            for test_class in test_class_names:
                print(test_class)
                suite.addTest(
                    unittest.defaultTestLoader.loadTestsFromName(
                        test_class, module=tests_classes
                    )
                )

            unittest.TextTestRunner().run(suite)



#!/usr/bin/env python
# coding: utf-8


"""
    python-creole utils
    ~~~~~~~~~~~~~~~~~~~    


    :copyleft: 2008-2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import re

from mycms.creole.html_parser.config import BLOCK_TAGS


strip_html_regex = re.compile(
    r"""
        \s*
        <
            (?P<end>/{0,1})       # end tag e.g.: </end>
            (?P<tag>[^ >]+)       # tag name
            .*?
            (?P<startend>/{0,1})  # closed tag e.g.: <closed />
        >
        \s*
    """,
    re.VERBOSE | re.MULTILINE | re.UNICODE,
)


def strip_html(html_code):
    """
    Delete whitespace from html code. Doesn't recordnize preformatted blocks!

    >>> strip_html(' <p>  one  \\n two  </p>')
    '<p>one two</p>'

    >>> strip_html('<p><strong><i>bold italics</i></strong></p>')
    '<p><strong><i>bold italics</i></strong></p>'

    >>> strip_html('<li>  Force  <br /> \\n linebreak </li>')
    '<li>Force<br />linebreak</li>'

    >>> strip_html('one  <i>two \\n <strong>   \\n  three  \\n  </strong></i>')
    'one <i>two <strong>three</strong> </i>'

    >>> strip_html('<p>a <unknown tag /> foobar  </p>')
    '<p>a <unknown tag /> foobar</p>'

    >>> strip_html('<p>a <pre> preformated area </pre> foo </p>')
    '<p>a<pre>preformated area</pre>foo</p>'

    >>> strip_html('<p>a <img src="/image.jpg" /> image.</p>')
    '<p>a <img src="/image.jpg" /> image.</p>'


    """

    def strip_tag(match):
        block = match.group(0)
        end_tag = match.group("end") in ("/", "/")
        startend_tag = match.group("startend") in ("/", "/")
        tag = match.group("tag")

        #        print("_"*40)
        #        print(match.groupdict())
        #        print("block.......: %r" % block)
        #        print("end_tag.....:", end_tag)
        #        print("startend_tag:", startend_tag)
        #        print("tag.........: %r" % tag)

        if tag in BLOCK_TAGS:
            return block.strip()

        space_start = block.startswith(" ")
        space_end = block.endswith(" ")

        result = block.strip()

        if end_tag:
            # It's a normal end tag e.g.: </strong>
            if space_start or space_end:
                result += " "
        elif startend_tag:
            # It's a closed start tag e.g.: <br />

            if space_start:  # there was space before the tag
                result = " " + result

            if space_end:  # there was space after the tag
                result += " "
        else:
            # a start tag e.g.: <strong>
            if space_start or space_end:
                result = " " + result

        return result

    data = html_code.strip()
    clean_data = " ".join([line.strip() for line in data.split("\n")])
    clean_data = strip_html_regex.sub(strip_tag, clean_data)
    return clean_data


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())



#!/usr/bin/env python
# coding: utf-8


"""
    python-creole
    ~~~~~~~~~~~~~


    :copyleft: 2008-2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

from xml.sax.saxutils import escape


def _mask_content(emitter, node, mask_tag):
    attrs = node.get_attrs_as_string()
    if attrs:
        attrs = " " + attrs

    tag_data = {
        "tag": node.kind,
        "attrs": attrs,
        "mask_tag": mask_tag,
    }

    content = emitter.emit_children(node)
    if not content:
        # single tag
        return "<<%(mask_tag)s>><%(tag)s%(attrs)s /><</%(mask_tag)s>>" % tag_data

    start_tag = "<<%(mask_tag)s>><%(tag)s%(attrs)s><</%(mask_tag)s>>" % tag_data
    end_tag = "<<%(mask_tag)s>></%(tag)s><</%(mask_tag)s>>" % tag_data

    return start_tag + content + end_tag


def raise_unknown_node(emitter, node):
    """
    unknown_emit callable for Html2CreoleEmitter
    
    Raise NotImplementedError on unknown tags.
    """
    content = emitter.emit_children(node)
    raise NotImplementedError(
        "Node from type '%s' is not implemented! (child content: %r)"
        % (node.kind, content)
    )


def use_html_macro(emitter, node):
    """
    unknown_emit callable for Html2CreoleEmitter
    
    Use the <<html>> macro to mask unknown tags.
    """
    return _mask_content(emitter, node, mask_tag="html")


def preformat_unknown_nodes(emitter, node):
    """
    Put unknown tags in a <pre> area.
    
    Usefull for html2textile.emitter.TextileEmitter()
    """
    return _mask_content(emitter, node, mask_tag="pre")


def escape_unknown_nodes(emitter, node):
    """
    unknown_emit callable for Html2CreoleEmitter
    
    All unknown tags should be escaped.
    """
    attrs = node.get_attrs_as_string()
    if attrs:
        attrs = " " + attrs

    tag_data = {
        "tag": node.kind,
        "attrs": attrs,
    }

    content = emitter.emit_children(node)
    if not content:
        # single tag
        return escape("<%(tag)s%(attrs)s />" % tag_data)

    start_tag = escape("<%(tag)s%(attrs)s>" % tag_data)
    end_tag = escape("</%(tag)s>" % tag_data)

    return start_tag + content + end_tag


def transparent_unknown_nodes(emitter, node):
    """
    unknown_emit callable for Html2CreoleEmitter 
    
    Remove all unknown html tags and show only
    their child nodes' content.
    """
    return emitter._emit_content(node)



from mycms.models import CMSEntries
from mycms.models import CMSPageTypes
from mycms.view_handlers.mycms_view import ViewObject

from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.db.utils import OperationalError
from mycms.view_handlers.mycms_view import ArticleList

from mycms.view_handlers import page_types

import logging

logger = logging.getLogger("mycms.page_handlers")


class MultiPageList(list):
    def __init__(self, total_entries):

        super(MultiPageList, self).__init__()
        self.total_entries = total_entries
        self.current_page_obj = None

    @property
    def page_range(self):
        """
        Returns the range of pages to display
        """
        return self

    @property
    def has_previous(self):
        """
        Returns boolean, True if there is a previous page
        and False if we are at the beginning.
        """

        if self.current_page_obj.page_number == 1:
            return False
        else:
            return True

    @property
    def page(self):
        """
        Returns 
        """
        pass

    @property
    def next_page(self):
        """
        Returns the next page. 
        """
        if self.current_page_obj.page_number < len(self) - 1:
            return self[self.current_page_obj.page_number + 1]
        pass

    @property
    def has_next(self):

        last_page_number = len(self)
        if self.current_page_obj.page_number < last_page_number:
            return True
        else:
            return False

    @property
    def has_previous(self):

        last_page_number = len(self)
        if self.current_page_obj.page_number > 1:
            return True
        else:
            return False

    @property
    def previous_page(self):
        if self.has_previous:
            obj = self[self.current_page_obj.page_number - 2]
            print(obj.title)
            return obj

        else:
            return None

    @property
    def next_page(self):
        if self.has_next:
            return self[self.current_page_obj.page_number - 2]


class MultiPage(object):
    def __init__(self, page_object, request=None):

        # page_object is a mycms.models.CMSEntries instance.
        # which represents the page.

        self.page_object = page_object
        self.page_number = 1  # This is always the first page.

        try:
            self.memberpageview_pagetype_obj = CMSPageTypes.objects.get(
                page_type="MEMBERPAGE"
            )

        except ObjectDoesNotExist as e:

            msg = "Could not load MEMBERPAGE view object. Going to create it."
            logger.debug(msg)
            self.memberpageview_pagetype_obj, _ = CMSPageTypes.objects.get_or_create(
                page_type="MEMBERPAGE",
                text="MemberPage Article",
                view_class="MemberPage",
                view_template="MemberPage.html",
            )

        except MultipleObjectsReturned as e:
            msg = "Got more than 1 CMSMultiPageType: MULTIPAGE. Database is inconsistent. Will return the first one."
            logger.info(msg)

            memberpageview_pagetype_obj = CMSMultiPageTypes.objects.filter(
                page_type="MEMBERPAGE"
            )[0]

    @property
    def is_multipage(self):
        return True

    @property
    def articles(self):
        """
        Return al the pages that have the same parent as us.
        """

        from django.db.models import Q

        # Get CMS entries starting at offset and give limit number results
        """
        Get the CMSEntries whose parent is the same as our parent. 
        Starting at the offset and give limit number of results. 
        """

        """
        This page_object is the parent of the sub articles. 
        Each cms entry has a path whose parent.path == the path of this
        page_object. We use this relation to find all children.
        """
        try:

            children = CMSEntries.objects.filter(
                Q(path__parent__path=self.page_object.path)
                & Q(published=True)
                & Q(lists_include=True)
            )
        except Exception as e:
            print(e)
        # need to know the total cmsentries in mycms.
        children_count = children.count()
        article_list = MultiPageList(total_entries=children_count)

        # We are the first in the list
        self.page_object.page_number = 1
        article_list.append(self.page_object)
        article_list.current_page_obj = self.page_object

        index = 1

        for obj in children:
            # BUG: We are just adding everythign into the article_list here!
            index = index + 1
            obj.page_number = index
            if self.page_object.id == obj.id:
                obj.current_page = True
                article_list.current_page_obj = obj
            else:
                obj.current_page = False
            article_list.append(obj)
        return article_list

    # def articles(self):
    # """Here we load all pages that says we are their parent."""

    # obj_list = CMSEntries.objects.filter(page_type = self.memberpageview_pagetype_obj,
    # path__parent__id = self.page_object.path.id)
    ##wrap the entries of the obj_list into their view_handler representations
    # view_list = []
    # for obj in obj_list:
    # view_list.append(ViewObject(page_object=obj))
    # return view_list

    # def page_types(self):
    # """
    # Refactor me into a parent class.
    # returns a list fo page_types
    # """
    # pagetype_objs = CMSPageTypes.objects.filter(page_type="MEMBERPAGE")

    # return pagetype_objs

    # def on_create(self):

    # print("Created a new article {}".format(self.page_object.title))

    # @property
    # def first_page_object(self):
    ##We are the first page object
    # return self.page_object

    # @property
    # def member_page_objects(self):

    # member_cmsentries =  CMSEntries.objects.filter(
    # path__parent=self.page_object.path).order_by('page_number')
    # return member_cmsentries

    # @property
    # def first_page_object(self):
    ##The first_page_object is always the parent. No need to do
    ##anything here since it is implemented in the CMSEntry

    # return self.page_object.parent

    # @property
    # def first_page(self):
    ##Legacy implementatin. THIS IS VERY MISLEADING. should be
    ##renamed as first_member_page.

    # entries = CMSEntries.objects.filter(
    # path__parent=self.page_object.path).order_by('page_number')

    # num_entries = len(entries)
    # print(num_entries)
    # if entries.count() > 0:
    # return entries[0]
    # else:
    # return None

    # @property
    # def has_first_page(self):

    # if self.first_page == None:
    # return False
    # else:
    # return True


class MemberPage(page_types.BasePage):
    def __init__(self, page_object, request=None):

        self.page_object = page_object
        self._page_number = 0

        self.last_member_page = 1

    def on_create(self):
        """
        A new member page has been created. Append it as the last page.

        """
        print("Created a memberpage called {}".format(self.page_object.title))
        my_parent_object = CMSEntries.objects.get(path=self.page_object.path.parent)

        print(my_parent_object.title)
        my_siblings = CMSEntries.objects.filter(
            path__parent=self.page_object.path.parent
        )

        for sibling in my_siblings:
            print(sibling.title)

        last_page_num = self.last_member_page_object.page_number
        self.page_object.page_number = last_page_num + 1
        self.page_object.save()

    # @property
    # def member_page_objects(self):

    # return CMSEntries.objects.filter(path__parent=self.page_object.path.parent).order_by('page_number')

    # @property
    # def first_member_page_object(self):
    # pass

    # @property
    # def last_member_page_object(self):

    # c = self.member_page_objects.count()

    # if c==1:
    # return self.member_page_objects[0]

    # elif c==0:
    # return None
    # else:
    ##We have more than one
    # return self.member_page_objects[c-1]

    # @property
    # def next_page(self):

    # """
    # Get the next page by getting all the pages that have greater page_number than us.
    # """

    # next_pages = CMSEntries.objects.filter(path__parent=self.page_object.path.parent, page_number__gt=self.page_object.page_number).order_by('page_number')
    # num_results = len(next_pages)
    # if  num_results > 0:
    # return next_pages[0]
    # else:
    # return None

    # @property
    # def previous_page(self):
    # previous_pages = CMSEntries.objects.filter(path__parent=self.page_object.path.parent, page_number__lt=self.page_object.page_number).order_by('-page_number')

    # num_results = len(previous_pages)
    # if num_results > 0:
    # return previous_pages[0]
    # else:
    # return None

    # @property
    # def first_page(self):

    # entries = CMSEntries.objects.filter(path__parent=self.page_object.path.parent).order_by('page_number')

    # return entries[0]

    # @property
    # def is_last_page(self):
    # entries = CMSEntries.objects.filter(path__parent=self.page_object.path.parent).order_by('-page_number')

    # if entries[0] == self.page_object:
    # return True
    # else:
    # return False

    # @property
    # def is_first_page(self):
    # if self.page_object == self.first_page:
    # return True
    # else:
    # return False
    @property
    def page_number(self):
        return self._page_number

    @page_number.setter
    def page_number(self, page_number):

        self._page_number = page_number

    @property
    def articles(self):
        """
        Return al the pages that have the same parent as us.
        """

        from django.db.models import Q

        # Get CMS entries starting at offset and give limit number results
        """
        Get the CMSEntries whose parent is the same as our parent. 
        Starting at the offset and give limit number of results. 
        """
        limit, offset, page = self.get_list_params()
        try:

            page_cmsentries = CMSEntries.objects.filter(
                Q(path__parent__id=self.page_object.parent().path.id)
                & Q(published=True)
                & Q(lists_include=True)
            )[offset : offset + limit]
        except Exception as e:
            # We hit here if there are no children
            print(e)
            pass

        # Get all the siblings of this page. Siblings are cmsentries that
        # have the same parent as this entry's parent.
        all_cmsentries = CMSEntries.objects.filter(
            Q(path__parent__id=self.page_object.parent().path.id)
            & Q(published=True)
            & Q(lists_include=True)
        )

        # need to know the total cmsentries in mycms.
        all_cmsentries_count = all_cmsentries.count()

        article_list = MultiPageList(total_entries=all_cmsentries_count)

        # insert the parent

        parent = CMSEntries.objects.get(path=self.page_object.parent().path)
        article_list.append(parent)
        index = 1

        for obj in page_cmsentries:
            # BUG: We are just adding everythign into the article_list here!
            index = index + 1
            obj.page_number = index
            if self.page_object.id == obj.id:
                obj.current_page = True
                article_list.current_page_obj = obj
            else:
                obj.current_page = False
            article_list.append(obj)
        return article_list



import requests
from bs4 import BeautifulSoup


url = "https://www.iconfinder.com/iconsets/tango-icon-library"


r = requests.get(url)
html = r.text

soup = BeautifulSoup(html)
icon_previews = soup.findAll("div", {"class": "icon-preview"})

for icon_preview in icon_previews:
    print("wget {}".format(icon_preview["data-preview-url"]))



class PageDoesNotExist(Exception):
    pass


class PageClassNotFound(Exception):
    pass


class PageActionNotFound(Exception):
    pass


class IncompatiblePageClass(Exception):
    pass


class PageExists(Exception):
    pass


class PathExists(Exception):
    pass



import unittest
from mycms.models import CMSEntries

from mycms.view_handlers.category_page import CategoryPage
from mycms.view_handlers.page_types import singlepageview_pagetype_obj
from mycms.view_handlers.page_types import multipageview_pagetype_obj
from mycms.view_handlers.page_types import allarticles_pagetype_obj


class CategoryPageTests(unittest.TestCase):
    def setUp(self):
        print("loaded {}".format(self.__class__.__name__))

    def test_all_sub_articles(self):

        # tests getting all sub articles of a particular category page
        page_object = CMSEntries.objects.get(path__path="/sysadmin/linux")

        x = CategoryPage(page_object)
        print("*" * 80)
        for i in x.all_sub_articles():
            print(i.title)

    def test_categories(self):

        page_object = CMSEntries.objects.get(path__path="/sysadmin/linux")
        x = CategoryPage(page_object)

        for i in x.categories:
            print(i.title)



from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

import logging
import pathlib
import os
from PIL import Image
import threading
import datetime
import faker
import random

from bs4 import BeautifulSoup
import simplejson as json
import threading
import arrow
from pathlib import Path
from django.conf import settings

from django.contrib.auth import logout
from django.shortcuts import render, render_to_response
from django.http import HttpResponse
from django.contrib.sitemaps import Sitemap
from django.http import JsonResponse
from django.http import HttpResponseRedirect
from django.http import HttpResponseServerError
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View
from django import forms
from django.forms.models import model_to_dict
from django.http import Http404
from django.http import HttpResponseServerError
from django.core.files.uploadedfile import UploadedFile
from django.utils.text import slugify
from django.views.generic.base import View
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.http import HttpResponse
from rest_framework import permissions
from django.contrib.auth import logout
from django.contrib.auth import login
from django.db.models import Q


from django.core.exceptions import ObjectDoesNotExist
from django.core.exceptions import MultipleObjectsReturned
from django.template import RequestContext

from django.http import HttpResponseNotFound

# Create your views here.

from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from rest_framework.views import exception_handler
from rest_framework.exceptions import APIException
from rest_framework.exceptions import NotFound


from rest_framework import authentication, permissions
from rest_framework.renderers import JSONRenderer
from rest_framework.parsers import JSONParser
from rest_framework import status

from rest_framework import filters
from rest_framework import generics

from django.views.decorators.csrf import csrf_exempt
from django.conf import settings

from django.http import HttpResponse
from django.views.generic import CreateView, DeleteView, ListView


from django_filters.rest_framework import DjangoFilterBackend

from loremipsum import get_paragraphs
from mycms.creole import creole2html

from mycms.serializers import CMSPageTypesSerializer
from mycms.serializers import CMSContentsSerializer
from mycms.serializers import CMSEntrySerializer
from mycms.serializers import CMSMarkUpSerializer
from mycms.serializers import CMSTemplatesSerializer
from mycms.serializers import CMSPathsSerializer
from mycms.serializers import CMSEntryExpandedSerializer
from mycms.serializers import LoremIpsumSerializer


from mycms.models import CMSPageTypes
from mycms.models import CMSContents
from mycms.models import CMSEntries
from mycms.models import CMSMarkUps
from mycms.models import CMSTemplates
from mycms.models import CMSPaths


from mycms.view_handlers import ViewObject


from mycms.api import CMSContentsViewSet
from mycms.api import CMSFormatterContent
from mycms import funclib

logger = logging.getLogger(name="mycms.views")

try:
    import wingdbstub
except:
    pass


def get_static_files_dir():
    """
    Gets the static files directory.
    """

    for each in settings.STATICFILES_DIRS:
        if each.endswith("static"):
            return each
    else:
        raise Exception(
            """Static Files Dir Not Found. Please create a 'static' dir within the parent path."""
        )


class CMSFileUpload(View):
    def get(self, request, **kwargs):

        format = request.GET.get("format", None)

        if format == "json":
            # Get a list of the image names.
            pass

        from django.conf import settings

        try:
            assets_dir = settings.YACMS_SETTINGS.get("ARTICLE_IMAGES_DIR")
        except AttributeError as e:
            print("ARTICLE_IMAGES_DIR IS NOT CONFIGURED! Falling back to default.")
            assets_dir = os.path.join(settings.BASE_DIR, "static/assets")

        path = kwargs.get("path", None).lstrip("/")
        fullpath = pathlib.Path(pathlib.Path(assets_dir), path)
        print(path)

        data = []

        if fullpath.exists():

            for each in fullpath.iterdir():
                if each.suffix in [".jpeg", ".jpg", ".gif", ".png", ".bmp"]:
                    data.append("/images/{}/{}".format(path, each.name))

        return JsonResponse(data, safe=False)
        # template = "mycms/FileUpload.html"
        # return render_to_response(template, {})

    def _mkdir(self, newdir):
        """Copied from http://code.activestate.com/recipes/82465-a-friendly-mkdir/ """
        """works the way a good mkdir should :)
            - already exists, silently complete
            - regular file in the way, raise an exception
            - parent directory(ies) does not exist, make them as well
        """
        if os.path.isdir(newdir):
            pass
        elif os.path.isfile(newdir):
            raise OSError(
                "a file with the same name as the desired "
                "dir, '%s', already exists." % newdir
            )
        else:
            head, tail = os.path.split(newdir)
            if head and not os.path.isdir(head):
                self._mkdir(head)
            # print "_mkdir %s" % repr(newdir)
            if tail:
                if not os.path.exists(newdir):
                    os.mkdir(newdir)

    def fileupload(self, request, **kwargs):

        mylock = threading.Lock()

        with mylock:
            from django.conf import settings

            """
            The files are uploaded to the assets directory. A directory is 
            created with the same path as the article that the asset is 
            uploaded to. For example an article at 
            /foo/bar/baz will hvae all images uploaded to 
            Path(settings.ASSETS_DIR, "foo/bar/baz/image)
            """

            try:
                # A sanity check to make sure the assets dir exists.
                # TODO: Decide if this code should really be here. IMHO the
                # test to make sure it exists should be elsewhere.
                assets_dir = settings.ASSETS_DIR

            except AttributeError as e:
                print("ASSETS_DIR IS NOT CONFIGURED! Falling back to default.")
                # TODO: Danger!! We assuming here that PARENT_DIR is defined.
                # What if the end user does not define it????
                assets_dir = os.path.join(settings.PARENT_DIR, "assets/")

            path = kwargs.get("path", None).lstrip("/")

            fullpath = pathlib.Path(pathlib.Path(assets_dir), path)

            if not fullpath.exists():
                self._mkdir(fullpath.as_posix())
            elif not fullpath.is_dir():
                # Fix this to return a proper json response.
                return HttpResponse("Error. Not full dir")

            thumbnailpath = pathlib.Path(fullpath, "thumbnails")

            if not thumbnailpath.exists():
                try:
                    os.makedirs(thumbnailpath.as_posix())
                except FileExistsError as e:
                    # If we hit here then it means, some other thread
                    # created it.
                    pass
            elif not thumbnailpath.is_dir():
                # Fix this to return a proper json response.
                return HttpResponse("Error. Not full dir")

            if request.method == "POST":

                uploaded_file = request.FILES.get("files[]")

                filename = uploaded_file.name

                p_filename = pathlib.Path(fullpath, filename)

                with open(p_filename.as_posix(), "wb+") as destination:
                    for chunk in uploaded_file.chunks():
                        destination.write(chunk)

                # Now do the thumbail.
                try:
                    size = 256, 256
                    outfile = pathlib.Path(thumbnailpath, filename)
                    im = Image.open(p_filename.as_posix())
                    im.thumbnail(size, Image.ANTIALIAS)
                    im.save(outfile.as_posix(), "JPEG")
                except IOError as e:
                    return JsonResponse(
                        {"error": "cannot create thumbnail for {}".format(outfile)}
                    )

                except Exception as e:
                    return JsonResponse(
                        {"error": "Unhandled exception: {}".format(outfile)}
                    )

    def post(self, request, **kwargs):

        self.fileupload(request, **kwargs)
        return HttpResponse("Completed")

    def delete(self, request, **kwargs):
        pass


def index(request, **kwargs):
    return HttpResponse("Index page")


def fileupload(request, **kwargs):

    template = "mycms/fileupload.html"
    return render_to_response(template, {})


from django.contrib.sitemaps import Sitemap

# from blog.models import Entry


class CMSSitemap(Sitemap):
    # changefreq = "never"
    priority = 0.9

    def items(self):
        return CMSEntries.objects.filter(published=True)

    def lastmod(self, obj):
        return obj.date_modified


# Django


class CMSLoginForm(forms.Form):
    username = forms.CharField(max_length=100, label="Username")
    password = forms.CharField(max_length=32, widget=forms.PasswordInput)


class CMSLoginView(View):
    def get(self, request, **kwargs):

        next_page = request.GET.get("next", "")

        if not request.user.is_authenticated:
            template_name = "mycms/Login.html"

            # also set a session variable to show the toolbar
            request.session["show_toolbar"] = True
            context = {"form": CMSLoginForm(), "next": next_page}
            return render(request, template_name, context)
        else:
            return HttpResponseRedirect("/profile")

    def post(self, request, **kwargs):

        username = request.POST.get("username", None)
        password = request.POST.get("password", None)

        # Set the next page to a default if not given.
        next_page = request.GET.get("next", "/cms/user/articles")

        # sanitize next_page

        if len(next_page) == 0:
            next_page = "/profile"

        if not (
            (next_page == "/")
            or next_page.startswith("/cms")
            or (next_page == "/profile")
        ):
            # We only allow redirects to cms contents or to /profile
            self.log.info("redirect after login denied to: {}".format(next_page))

        from django.contrib.auth import authenticate

        user = authenticate(username=username, password=password)

        error_msg = ""
        if user is not None:
            # the password verified for the user
            if user.is_active:
                login(request, user)
                if next_page is not None:
                    return HttpResponseRedirect(next_page)
                else:
                    # Go to the user's account page.
                    return HttpResponse(content=b"You are now logged in.")
            else:
                print("The password is valid, but the account has been disabled!")
                error_msg = "Account is disabled."
        else:
            # the authentication system was unable to verify the username and password
            error_msg = "The system was unable to verify the username and password."

        template_name = "mycms/Login.html"
        return render(request, template_name, {"error_msg": error_msg})


class CMSUserProfileView(View):
    def get(self, request, **kwargs):
        template_name = "mycms/Profile.html"
        if not request.user.is_authenticated:
            return HttpResponseRedirect("/login")
        else:
            return render(request, template_name)


class CMSUserAdminPagesView(View):
    def get(self, request, **kwargs):
        pass


class CMSLogoutView(View):
    def get(self, request, **kwargs):
        logout(request)
        template_name = "mycms/Logout.html"

        # So lets log the user out.

        if request.user.is_active:
            logout(request)

        return HttpResponseRedirect("/login/")


class CMSFrontPage(View):
    def get(self, request, **kwargs):

        template_name = "mycms/Index.html"

        # TODO: Fix this hack. We create a fake view_object for the frontpage
        # so that we can pass a few template data.

        class View_Object:
            pass

        view_object = View_Object()

        if settings.FORCE_SHOW_ADVERTS or (settings.DEBUG == False):
            view_object.SHOW_ADVERTS = True
        else:
            print("SHOW_ADDS_IS FALSE")
            view_object.SHOW_ADVERTS = False

        print(
            "FORCE_SHOW_ADVERTS={},DEBUG={},SHOW_ADDS={}".format(
                settings.FORCE_SHOW_ADVERTS, settings.DEBUG, view_object.SHOW_ADVERTS
            )
        )
        return render(request, template_name, {"view_object": view_object})


class LoremIpsumAPIView(APIView):

    """Returns loremipsum paragraphs"""

    authentication_classes = (
        authentication.SessionAuthentication,
        authentication.TokenAuthentication,
    )
    permission_classes = (IsAuthenticated,)

    def get(self, request, **kwargs):

        data = {"message": "You need to send a post."}
        return Response(data, status=status.HTTP_200_OK)

    def post(self, request, **kwargs):
        """Get X number of paragraphs"""
        serializer = LoremIpsumSerializer(data=request.data)
        if serializer.is_valid():
            num_paragraphs = request.data.get("num_paragraphs")

            paragraphs_list = get_paragraphs(int(num_paragraphs))

            html = ""
            for paragraph in paragraphs_list:
                html += "{}\n\n".format(paragraph)

            data = {"content": html}
            return Response(data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class CMSPathsAPIView(APIView):
    """
    View to list PageTypes handled by the system
    """

    authentication_classes = (
        authentication.SessionAuthentication,
        authentication.TokenAuthentication,
    )
    permission_classes = (IsAuthenticated,)

    def get(self, request, **kwargs):
        """
        Get a list of all paths or info about a path
        """

        format = kwargs.get("format", None)
        resource_id = kwargs.get("resource_id", None)

        if resource_id:
            # We are asking for a single entry
            try:
                cmspath = CMSPaths.objects.get(pk=resource_id)
            except ObjectDoesNotExist as e:
                msg_dict = {
                    "error": "Resource with id {} does not exist".format(resource_id)
                }
                return Response(data=msg_dict, exception=True, status=200)

            serializer = CMSPathsSerializer(cmspath, many=False)
            return Response(serializer.data)

        paths = CMSPaths.objects.all()
        serializer = CMSPathsSerializer(paths, many=True)
        return Response(serializer.data)

    def post(self, request, format=None, **kwargs):

        # Fix the path if it does not start with a / by appending
        # it to the parent path.

        path_str = request.data.get("path")
        parent_str = request.data.get("parent")

        # parent_obj = CMSPaths.objects.get(parent__path=parent_str)

        data_dict = request.data.dict()

        # if path_str and (not path_str.startswith("/")):

        # if int(parent_obj.id) != 1: #1 is always / so we need to get only what is not 1
        # parent_cmspath = CMSPaths.objects.get(pk=parent_id)
        # data_dict["path"] = "{}/{}".format(parent_cmspath.path, path_str)
        # else:
        # data_dict["path"] = "/{}".format(path_str)

        if (parent_str == "") or (parent_str == "/"):
            data_dict["path"] = data_dict["path"] = "/{}".format(path_str)
        else:
            data_dict["path"] = "{}/{}".format(parent_str, path_str)

        parent = CMSPaths.objects.get(path=parent_str)
        data_dict["parent"] = parent.id
        serializer = CMSPathsSerializer(data=data_dict)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class CMSPageTypesAPIView(APIView):
    """
    View to list PageTypes handled by the system

    * Requires token authentication.
    * Only admin users are able to access this view.
    """

    # authentication_classes = (authentication.TokenAuthentication,)
    permission_classes = (IsAuthenticated,)

    def get(self, request, **kwargs):
        """
        Get a list of all available PageTypes
        """

        format = kwargs.get("format", None)

        pagetypes = CMSPageTypes.objects.all()
        serializer = CMSPageTypesSerializer(pagetypes, many=True)
        return Response(serializer.data)

    def post(self, request, format=None):
        serializer = CMSPageTypesSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class CMSMarkUpsAPIView(APIView):
    """
    View to list PageTypes handled by the system

    * Requires token authentication.
    * Only admin users are able to access this view.
    """

    authentication_classes = (
        authentication.SessionAuthentication,
        authentication.TokenAuthentication,
    )
    permission_classes = (IsAuthenticated,)

    def get(self, request, **kwargs):
        """
        Get a list of all available PageTypes
        """

        format = kwargs.get("format", None)
        pagetypes = CMSMarkUps.objects.all()
        serializer = CMSMarkUpSerializer(pagetypes, many=True)
        return Response(serializer.data)

    def post(self, request, format=None):
        serializer = CMSMarkUpSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class CMSTemplatesAPIView(APIView):
    """
    View to list PageTypes handled by the system

    * Requires token authentication.
    * Only admin users are able to access this view.
    """

    # authentication_classes = (authentication.TokenAuthentication,)
    permission_classes = (IsAuthenticated,)

    def get(self, request, **kwargs):
        """
        Get a list of all available PageTypes
        """

        format = kwargs.get("format", None)
        pagetypes = CMSTemplates.objects.all()
        serializer = CMSTemplatesSerializer(pagetypes, many=True)
        return Response(serializer.data)

    def post(self, request, format=None):
        serializer = CMSTemplatesSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


import django_filters


class CMSEntriesFilter(django_filters.FilterSet):
    class Meta:
        model = CMSEntries
        fields = [
            "id",
            "page_type",
            "slug",
            "date_created",
            "published",
            "frontpage",
            "date_created",
        ]


class CMSEntriesROAPIView(generics.ListAPIView):

    queryset = CMSEntries.objects.all()
    serializer_class = CMSEntrySerializer
    filter_class = CMSEntriesFilter
    permission_classes = (IsAuthenticated,)


from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from rest_framework.permissions import IsAuthenticated


class CMSEntriesAPIView(APIView):
    """
    View to list PageTypes handled by the system

    * Requires token authentication.
    * Only admin users are able to access this view.
    """

    authentication_classes = (SessionAuthentication, BasicAuthentication)
    permission_classes = (IsAuthenticated,)

    filter_backends = (DjangoFilterBackend,)

    def get(self, request, **kwargs):
        """
        Get a list of all available PageTypes
        """
        format = kwargs.get("format", None)
        resource_id = kwargs.get("resource_id")

        parent_id = self.request.query_params.get("parent", None)
        page_id = self.request.query_params.get("id", None)
        page_type_id = self.request.query_params.get("page_type", None)
        expand = self.request.query_params.get("expand", None)

        if resource_id:
            # We are asking for a single entry

            try:
                cmsentry = CMSEntries.objects.get(pk=resource_id)
            except ObjectDoesNotExist as e:
                msg_dict = {
                    "error": "Resource with id {} does not exist".format(resource_id)
                }
                return Response(data=msg_dict, exception=True, status=200)

            serializer = CMSEntrySerializer(cmsentry, many=False)
            return Response(serializer.data)

        cmsentries = CMSEntries.objects.all()

        if parent_id is not None:

            # We are receiving the id of the parent. We want to return all
            # the cms entries whose path is the children of the path
            # of the parent.

            parent_cmsentry = CMSEntries.objects.get(id=parent_id)
            parent_path = CMSPaths.objects.get(id=parent_id)

            cmsentries = cmsentries.filter(path__parent=parent_cmsentry.path)

        if page_type_id:
            cmsentries = cmsentries.filter(page_type=page_type_id)

        if page_id is not None:
            cmsentries = cmsentries.filter(id=page_id)

        if expand:
            serializer = CMSEntryExpandedSerializer(cmsentries, many=True)
        else:
            serializer = CMSEntrySerializer(cmsentries, many=True)

        return Response(serializer.data)

    def post(self, request, format=None):

        # Because the serializer is so fucking picky, we don't use it
        # to serialize the form

        title = request.data.get("title", None)
        path_id = request.data.get("path", None)
        slug = request.data.get("slug", None)
        page_type_id = request.data.get("page_type", None)

        if slug is None:
            slug = slugify(title)

        if None in (title, path_id, slug, page_type_id):
            return Response(
                {"error": "title,path, slug and page_type are not optional"},
                status=status.HTTP_400_BAD_REQUEST,
            )

        try:
            cmspath_obj = CMSPaths.objects.get(id=path_id)
        except ObjectDoesNotExist as e:
            return Response(
                {"error": "CMSPath: {} does not exist".format(path_id)},
                status=status.HTTP_400_BAD_REQUEST,
            )

        try:
            cmspagetype_obj = CMSPageTypes.objects.get(id=page_type_id)
        except ObjectDoesNotExist as e:
            return Response(
                {"error": "CMSPageType: {} does not exist".format(page_type_id)},
                status=status.HTTP_400_BAD_REQUEST,
            )

        # Now create the cms_obj.
        new_cmsentry = CMSEntries()
        new_cmsentry.title = title
        new_cmsentry.path = cmspath_obj
        new_cmsentry.slug = slug
        new_cmsentry.page_type = cmspagetype_obj
        new_cmsentry.save()

        content = CMSContents()
        content.content = "No content. Edit me."
        content.save()

        new_cmsentry.content.add(content)
        new_cmsentry.on_create()

        return Response(
            model_to_dict(new_cmsentry, exclude="content"), status=status.HTTP_200_OK
        )

    def put(self, request, **kwargs):
        serializer = CMSEntrySerializer(data=request.data)

        format = kwargs.get("format", None)
        expand = self.request.query_params.get("expand", None)

        id = request.data.get("id", None)
        if id:
            id = int(id)

        if not id:
            res = {"code": 400, "message": "PUT request requires an id parameter"}
            return Response(data=json.dumps(res), status=status.HTTP_400_BAD_REQUEST)

        cmsentry_object = CMSEntries.objects.get(id=id)

        # Now update the cmsentry_object with the attributes from
        # the request.data

        for key in request.data:

            if key == "date_created_epoch":
                print("Recieved a date_created_epoch")
                value = request.data.get("date_created_epoch")
                created_datetime = datetime.datetime.utcfromtimestamp(int(value) / 1000)

                cmsentry_object.date_created = created_datetime

            elif key == "date_created_str":
                # WE re
                x1 = request.data.get("date_created_str")
                x2 = arrow.get(x1, "YYYY/MM/DD HH:mm")
                setattr(cmsentry_object, "date_created", str(x2))

            elif key == "date_modified_str":
                y1 = request.data.get("date_modified_str")
                y2 = arrow.get(y1, "YYYY/MM/DD HH:mm")
                setattr(cmsentry_object, "date_modified", str(y2))

            elif hasattr(cmsentry_object, key) and (key != "id"):

                value = request.data.get(key)
                if value == "true":
                    value = True
                if value == "false":
                    value = False
                setattr(cmsentry_object, key, value)

        cmsentry_object.save()

        print(cmsentry_object.frontpage)

        # if expand:

        # We return only the values that we have set.

        return Response(request.data, status=status.HTTP_200_OK)

        # else:
        #    return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class CMSContentsAPIView(APIView):
    """
    View to list PageTypes handled by the system

    * Requires token authentication.
    * Only admin users are able to access this view.
    """

    # authentication_classes = (authentication.TokenAuthentication,)
    permission_classes = (IsAuthenticated,)

    def get(self, request, **kwargs):
        """
        Get a list of all available PageTypes
        """

        format = kwargs.get("format", None)
        pagetypes = CMSContents.objects.all()
        content_id = self.request.query_params.get("resource_id", None)

        contents = CMSContents.objects.all()

        if content_id is not None:
            contents = contents.filter(id=content_id)

        serializer = CMSContentsSerializer(contents, many=True)
        return Response(serializer.data)

    def post(self, request, format=None):
        serializer = CMSContentsSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()

            print(serializer.data)

            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def put(self, request, format=None):

        serializer = CMSContentsSerializer(data=request.data)

        resource_id = request.data.get("id", None)

        if not resource_id:
            res = {
                "code": 400,
                "message": "PUT request requires a resource id parameter",
            }
            return Response(data=res, status=status.HTTP_200_OK)

        if serializer.is_valid():
            cmscontent_object = CMSContents.objects.get(id=resource_id)
            cmscontent_object.content = request.data.get("content")
            cmscontent_object.save()

            include_html = request.GET.get("include_html", None)

            if include_html:
                # Custom pack results:
                cmscontent_dict = model_to_dict(cmscontent_object)

                # We need to get the html. So We need the freaking ViewObject.

                cmscontent_dict["html"] = cmscontent_object.html

                return Response(cmscontent_dict, status=status.HTTP_202_ACCEPTED)
            else:
                serializer = CMSContentsSerializer(cmscontent_object)
                return Response(serializer.data, status=status.HTTP_202_ACCEPTED)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class CMSIndexView(APIView):
    """
    The index view of a YACMS website.

    url: /cms.

    This is a special page since it needs to exist before any other
    categories or pages can be created.

    To create a new page , we need at minimum to post

    title:
    page_type:

    """

    def get(self, request, **kwargs):
        pass


#class CMSArticleListsView(View):
    #"""
    #Implements a paginated list view of all published articles. 
    #"""

    #from django.db.models import Q

    #def get(self, request, **kwargs):

        #obj_list = CMSEntries.objects.filter(
            #(
                #Q(page_type=singlepageview_pagetype_obj)
                #| Q(page_type=multipageview_pagetype_obj)
            #)
            #& Q(path__parent__id=self.page_object.path.id)
        #)[start : start + page_size]
        ## wrap the entries of the obj_list into their view_handler representations
        #view_list = []

        #for obj in obj_list:
            #view_list.append(ViewObject(page_object=obj))

        #return view_list


class CMSPageView(View):
    """
    The main interface to the website. This view serves all of the web pages
    except pages served by:
    
        CMSArticleListsView.    
        CMSIndexView
        CMSFrontPage
    """

    def get_object(self, request, **kwargs):
        """
        This method gets the cmsentries object from the database. 
        """

        path = kwargs.get("path", None)
        page_id = kwargs.get("page_id", None)

        """
        We use the path to search for the object if it is provided. Otherwise
        we can also load the page via page_id. If none is provided then 
        we just want to load the root. 
        """

        try:
            if path:
                obj = ViewObject(path=path, request=request)

            elif page_id:
                obj = ViewObject(page_id=page_id, request=request)
            else:
                """
                If no path or page_id is given then we just load /
                The path /  is the global cmsentries list which is 
                the root of the CMS. This root is created by default on 
                first access.  It is created as ALLARTICLES page_type.
                """
                obj = ViewObject(path="/", request=request)

            return obj

        except ObjectDoesNotExist as e:
            """
            Handle special case where we are in the root of the CMS i.e. /cms/
            """
            if path in [None, "/", "about"]:
                """
                / and /about are special pages in that they shoud be created
                if they do not yet exist. 
                """
                if path is None:
                    path = "/"
                elif not path.startswith("/"):
                    path = "/" + path

                """This code is obsolete. we are now creating all the 
                required pagetypes on first import of mycms.view_handlers.page_types"""

                # """
                path_obj, _ = CMSPaths.objects.get_or_create(path=path)

                if path == "/about":
                    page_type = "SINGLEPAGE"
                    title = "About"
                    text = funclib.get_fake_contents(random.randint(3, 5))
                    slug = "about"
                else:
                    page_type = "ALLARTICLES"
                    title = "MyCMS - Yet Another CMS"
                    text = "MYCMS Main Page Content. This is not shown in a default Category Page Template"
                    slug = ""
                try:
                    pagetype_obj, _ = CMSPageTypes.objects.get_or_create(
                        page_type=page_type
                    )

                except Exception as e:

                    logger.warn("Multiple PageType {} found in database")
                    return HttpResponseServerError(
                        "Server Error: Your favourite devops is getting woken up."
                    )

                try:
                    entry_obj, c = CMSEntries.objects.get_or_create(
                        page_type=pagetype_obj, path=path_obj, title=title, slug=slug
                    )

                    if c:
                        # We also need to create a content object to attach to entry_obj
                        content_object = CMSContents()
                        content_object.content = text
                        content_object.meta_description = "myCMS - A django based cms"

                        markup_obj, c = CMSMarkUps.objects.get_or_create(
                            markup="Creole"
                        )
                        content_object.markup = markup_obj

                        content_object.save()
                        entry_obj.content.add(content_object)
                        entry_obj.save()

                except MultipleObjectsReturned as e:
                    msg = "Multiple CMSEntries for /cms found. Database is inconsistent. Using the first one found. "
                    logger.warn(msg)

                    entry_obj = CMSEntries.objects.filter(path=path_obj)[0]
                obj = ViewObject(path=path)
                return obj
            else:

                # TODO: Make sure to log this too
                raise Http404("Page Does Not Exist.")

    def get(self, request, **kwargs):
        """
        The main entry point to the frontend of the CMS. 
        All website user pages are obtained through this method."""

        get = request.GET

        """
        Whenever ?toolbar is added to the get parameter, we save it to the session
        This is used as a flag to show the files. 
        """
        toolbar = request.GET.get("toolbar", None)
        if toolbar and toolbar.upper() == "TRUE":
            request.session["show_toolbar"] = True
        elif toolbar and toolbar.upper() == "FALSE":
            request.session["show_toolbar"] = False

        login = request.GET.get("login", None)
        logout = request.GET.get("logout", None)

        if login:
            """
            When a page is requested with ?login=X where X can be anything, we 
            redirect them to the login page which will send it back to this 
            page once logged in. 
            """

            # But only do it if user is not yet logged in
            if not request.user.is_authenticated:
                # so redirecto user to log in page and give our path to come back to
                return HttpResponseRedirect("/cms/login/?next={}".format(request.path))

            else:
                """
                The user is not logged in. 
                """

                ###
                # TODO: Implement a message to the user that he is already logged in.
                ###

                # For now do dnothing
                pass

        if logout and request.user.is_authenticated:
            # jist logout the current user and send him to the logout.
            return HttpResponseRedirect("/cms/logout/")

        # get object will load return an instance of mycms.view_handlers.ViewObject
        # which encapsulates the CMSEntry and all other operations on it.
        obj = self.get_object(request, **kwargs)
        obj.request = request

        # Save the toolbar value in the session in the show_toolbar member variable so
        # that it can be accessible in the web page template code.
        obj.show_toolbar = request.session.get("show_toolbar", False)

        template = obj.template

        if settings.DEBUG:
            from mycms.view_handlers import page_types

            return render_to_response(template, {"view_object": obj})
        else:

            try:
                return render_to_response(template, {"view_object": obj})

            except Exception as e:

                # if we are in debug mode then show then let django handle showing
                # the debug error:
                msg = "Application Error {}".format(e)
                return HttpResponse(content=bytes(msg, "utf-8"), status=500)

    def post(self, request, **kwargs):
        print(request, kwargs)
        return HttpResponse("Not implemented")

    def put(self, request, **kwargs):
        print(request, kwargs)
        return HttpResponse("Not Implemented")


########################################################################
class AssetsUploaderView(View):
    """Handles the uploading and deleting of images. Uses multiuploader AJAX plugin.
    made from api on: https://github.com/blueimp/jQuery-File-Upload
    """

    @csrf_exempt
    def dispatch(self, *args, **kwargs):
        return super(AssetsUploaderView, self).dispatch(*args, **kwargs)

    def _mkdir(self, newdir):
        """Copied from http://code.activestate.com/recipes/82465-a-friendly-mkdir/ """
        """works the way a good mkdir should :)
            - already exists, silently complete
            - regular file in the way, raise an exception
            - parent directory(ies) does not exist, make them as well
        """
        if os.path.isdir(newdir):
            pass
        elif os.path.isfile(newdir):
            raise OSError(
                "a file with the same name as the desired "
                "dir, '%s', already exists." % newdir
            )
        else:
            head, tail = os.path.split(newdir)
            if head and not os.path.isdir(head):
                try:
                    self._mkdir(head)
                except FileExistsError as e:
                    pass

            # print "_mkdir %s" % repr(newdir)
            if tail:
                try:
                    os.mkdir(newdir)
                except FileExistsError as e:
                    pass

    # ----------------------------------------------------------------------
    def get(self, request, **kwargs):

        """
        Creates the javascript to load the Krajee file upload.
        
        This needs to return a configuration like
        
        
        {
            error: 'An error exception message if applicable',
            errorkeys: [], // array of thumbnail keys/identifiers that have errored out (set via key property in initialPreviewConfig
            initialPreview: [
            ], // initial preview configuration 
            initialPreviewConfig: [
                // initial preview configuration if you directly want initial preview to be displayed with server upload data
            ],
            initialPreviewThumbTags: [
                // initial preview thumbnail tags configuration that will be replaced dynamically while rendering
            ],
            append: true // whether to append content to the initial preview (or set false to overwrite)
        }
        """

        assets_dir = self.get_assets_dir()

        path = kwargs.get("path", "").lstrip("/")
        fullpath = pathlib.Path(pathlib.Path(assets_dir), path)

        initialPreviewConfig = []
        initialPreview = []
        key = 0

        print(fullpath.as_posix())
        try:
            filenames = os.listdir(fullpath.as_posix())
        except OSError as e:
            # This means that there is no directory yet. Nothing uploaded
            # at all
            return JsonResponse(
                {
                    "initialPreviewConfig": initialPreviewConfig,
                    "initialPreview": initialPreview,
                }
            )

        for filename in filenames:

            p_filename = pathlib.Path(fullpath, filename)

            if not p_filename.is_dir():

                stat = os.stat(p_filename.as_posix())
                image_url = url = "/static/assets/{}/{}".format(path, filename)
                thumbnail_url = "/static/assets/{}/thumbnails/{}".format(path, filename)
                delete_url = "/cms/{}/assets_manager/{}".format(path, filename)

                print("*" * 80)

                initial_pconf_entry = {
                    "caption": filename,
                    "size": stat.st_size,
                    "width": "512px",
                    "url": request.path,
                    "key": filename,
                }

                key = key + 1

                initialPreviewConfig.append(initial_pconf_entry)
                initialPreview.append(image_url)

        return JsonResponse(
            {
                "initialPreviewConfig": initialPreviewConfig,
                "initialPreview": initialPreview,
                "append": False,
            }
        )

    def get_assets_dir(self):
        try:
            assets_dir = settings.ASSETS_DIR
        except AttributeError as e:
            print("ASSETS_DIR IS NOT CONFIGURED! Falling back to default.")
            assets_dir = os.path.join(settings.BASE_DIR, "static/assets")
        return assets_dir

    # ----------------------------------------------------------------------
    def post(self, request, *args, **kwargs):
        """"""

        filename = request.POST.get("key", None)
        print(request.GET)
        print(request.POST)

        if filename is not None:

            # if we are getting a "{'key': ['0']} in the request.POST then it
            # means the user wants to delete a file.
            return self.filedelete(kwargs, filename)

        else:
            # if we are getting a {'file_id': ['0']}> then it means its a fileupload
            return self.fileupload(request, *args, **kwargs)

    def filedelete(self, kwargs, filename):
        path = kwargs.get("path", None).lstrip("/")
        fullpath = pathlib.Path(pathlib.Path(self.get_assets_dir()), path)

        p_filename = pathlib.Path(fullpath, filename)

        logger.debug("Going to delete, {}".format(p_filename.as_posix()))

        try:
            os.remove(p_filename.as_posix())
            return JsonResponse({})

        except Exception as e:

            return JsonResponse(
                {"error": "Failed to delete"}, status=status.HTTP_410_GONE
            )

    def fileupload(self, request, *args, **kwargs):

        mylock = threading.Lock()
        print(request.GET)
        print(request.POST)

        with mylock:
            from django.conf import settings

            assets_dir = self.get_assets_dir()
            path = kwargs.get("path", None).lstrip("/")

            fullpath = pathlib.Path(pathlib.Path(assets_dir), path)

            if not fullpath.exists():
                self._mkdir(fullpath.as_posix())
            elif not fullpath.is_dir():
                # Fix this to return a proper json response.
                return HttpResponse("Error. Not full dir")

            thumbnailpath = pathlib.Path(fullpath, "thumbnails")

            if not thumbnailpath.exists():
                try:
                    os.makedirs(thumbnailpath.as_posix())
                except FileExistsError as e:
                    # It must have been created by another thread.
                    pass
            elif not thumbnailpath.is_dir():
                # Fix this to return a proper json response.
                return HttpResponse("Error. Not full dir")

            if request.method == "POST":
                uploaded_file = request.FILES.get("file_data")

                filename = uploaded_file.name

                p_filename = pathlib.Path(fullpath, filename)

                with open(p_filename.as_posix(), "wb+") as destination:
                    for chunk in uploaded_file.chunks():
                        destination.write(chunk)

                # Now do the thumbail.
                try:
                    size = 256, 256
                    outfile = pathlib.Path(thumbnailpath, filename)
                    im = Image.open(p_filename.as_posix())
                    im.thumbnail(size, Image.ANTIALIAS)
                    im.save(outfile.as_posix())
                except IOError as e:
                    print(e)
                    return JsonResponse(
                        {"error": "cannot create thumbnail for {}".format(outfile)}
                    )

                except Exception as e:
                    return JsonResponse(
                        {"error": "Unhandled exception: {}".format(outfile)}
                    )

                stat = os.stat(p_filename.as_posix())
                image_url = url = "/static/assets/{}/{}".format(path, filename)
                thumbnail_url = "/static/assets/{}/thumbnails/{}".format(path, filename)
                delete_url = "/cms/{}/assets_manager/{}".format(path, filename)

                files = []
                file_dict = {
                    "name": filename,
                    "size": stat.st_size,
                    "url": image_url,
                    "thumbnailUrl": thumbnail_url,
                    "deleteUrl": delete_url,
                    "deleteType": "DELETE",
                }

                initial_pconf_entry = {
                    "caption": filename,
                    "size": 1024,
                    "width": "512px",
                    "url": image_url,
                    "key": filename,
                }

                initialPreviewConfig = []

                files.append(file_dict)

                # return JsonResponse({'files': files})
                return self.get(request, *args, **kwargs)

    # ----------------------------------------------------------------------
    def delete(self, request, **kwargs):
        """"""
        path = kwargs.get("path", None).lstrip("/")
        filename = kwargs.get("filename", None).lstrip("/")
        fullpath = pathlib.Path(pathlib.Path(self.get_assets_dir()), path)

        p_filename = pathlib.Path(fullpath, filename)

        logger.debug("Going to delete, {}".format(p_filename.as_posix()))

        """We need to return a format as below:
            {"files": [
              {
                "picture1.jpg": true
              },
              {
                "picture2.jpg": true
              }
            ]}
            """

        try:

            os.remove(p_filename.as_posix())

            files = [{filename: True}]

            return JsonResponse({"files": files})

        except Exception as e:

            return HttpResponse("Deleted")


# class PictureCreateView(CreateView):
# model = Picture
# fields = "__all__"

# def form_valid(self, form):
# self.object = form.save()
# files = [serialize(self.object)]
# data = {'files': files}
# response = JSONResponse(data, mimetype=response_mimetype(self.request))
# response['Content-Disposition'] = 'inline; filename=files.json'
# return response

# def form_invalid(self, form):
# data = json.dumps(form.errors)
# return HttpResponse(content=data, status=400, content_type='application/json')

# class BasicVersionCreateView(PictureCreateView):
# template_name_suffix = '_basic_form'


# class BasicPlusVersionCreateView(PictureCreateView):
# template_name_suffix = '_basicplus_form'


# class AngularVersionCreateView(PictureCreateView):
# template_name_suffix = '_angular_form'


# class jQueryVersionCreateView(PictureCreateView):
# template_name_suffix = '_jquery_form'


# class PictureDeleteView(DeleteView):
# model = Picture

# def delete(self, request, *args, **kwargs):
# self.object = self.get_object()
# self.object.delete()
# response = JSONResponse(True, mimetype=response_mimetype(request))
# response['Content-Disposition'] = 'inline; filename=files.json'
# return response


# class PictureListView(ListView):
# model = Picture

# def render_to_response(self, context, **response_kwargs):
# files = [ serialize(p) for p in self.get_queryset() ]
# data = {'files': files}
# response = JSONResponse(data, mimetype=response_mimetype(self.request))
# response['Content-Disposition'] = 'inline; filename=files.json'
# return response


from django.http import HttpResponse
from django.shortcuts import render
from django.views import View
import os
from django.template import loader, Context


class MockupLoader(View):
    def get(self, request, **kwargs):

        template_name = kwargs.get("template", None)

        if template_name:
            return render(request, "{}".format(template_name))
        else:
            # Return a list of html files in bootstrap_templates

            mockups_dir = os.path.join(settings.BASE_DIR, "mockups/")

            files = []
            for filename in os.listdir(path=mockups_dir):
                if filename.endswith(".html"):
                    files.append(filename)

            return render(request, "mockups_index.html", {"files": files})


class LogoViewer(View):

    pass


class CMSPreviewAPIView(View):
    def get(self, request):
        raise NotImplementedError("This method is not supported")

    def post(self, request):
        # We expect to recieve unformatted text and we shall
        # return it formatted.

        # For now it is assumed this is Creole
        pass


class CMSEntriesContentAreaFilter(forms.Form):
    pass


class CMSUserContentArea(View):
    """This is a list view with filter."""

    def get(self, request):

        return render(request, "mycms/CMSUserContentArea.html")



from django import forms
from django.db import models
from django.template.loader import render_to_string
from django.forms.widgets import Select, MultiWidget, DateInput, TextInput
from time import strftime


class JqSplitDateTimeWidget(MultiWidget):
    def __init__(self, attrs=None, date_format=None, time_format=None):
        date_class = attrs["date_class"]
        time_class = attrs["time_class"]
        del attrs["date_class"]
        del attrs["time_class"]

        time_attrs = attrs.copy()
        time_attrs["class"] = time_class
        date_attrs = attrs.copy()
        date_attrs["class"] = date_class

        widgets = (
            DateInput(attrs=date_attrs, format=date_format),
            TextInput(attrs=time_attrs),
            TextInput(attrs=time_attrs),
            Select(attrs=attrs, choices=[("AM", "AM"), ("PM", "PM")]),
        )

        super(JqSplitDateTimeWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        if value:
            d = strftime("%Y-%m-%d", value.timetuple())
            hour = strftime("%I", value.timetuple())
            minute = strftime("%M", value.timetuple())
            meridian = strftime("%p", value.timetuple())
            return (d, hour, minute, meridian)
        else:
            return (None, None, None, None)

    def format_output(self, rendered_widgets):
        """
        Given a list of rendered widgets (as strings), it inserts an HTML
        linebreak between them.
        
        Returns a Unicode string representing the HTML for the whole lot.
        """
        return "Date: %s<br/>Time: %s:%s %s" % (
            rendered_widgets[0],
            rendered_widgets[1],
            rendered_widgets[2],
            rendered_widgets[3],
        )

    class Media:
        css = {}
        js = ("js/jqsplitdatetime.js",)



# coding: utf-8


"""
    Creole macros
    ~~~~~~~~~~~~~

    Note: all mecro functions must return unicode!

    :copyleft: 2008-2014 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.

"""
from __future__ import division, absolute_import, print_function, unicode_literals

from xml.sax.saxutils import escape

try:
    from pygments import highlight

    PYGMENTS = True
except ImportError:
    PYGMENTS = False

from mycms.creole.shared.utils import get_pygments_lexer, get_pygments_formatter


def html(text):
    """
    Macro tag <<html>>...<</html>>
    Pass-trought for html code (or other stuff)
    """
    return text


def pre(text):
    """
    Macro tag <<pre>>...<</pre>>.
    Put text between html pre tag.
    """
    return "<pre>%s</pre>" % escape(text)


def code(ext, text):
    """
    Macro tag <<code ext=".some_extension">>...<</code>>
    If pygments is present, highlight the text according to the extension.
    """
    if not PYGMENTS:
        return pre(text)

    try:
        source_type = ""
        if "." in ext:
            source_type = ext.strip().split(".")[1]
        else:
            source_type = ext.strip()
    except IndexError:
        source_type = ""

    lexer = get_pygments_lexer(source_type, code)
    formatter = get_pygments_formatter()

    try:
        highlighted_text = highlight(text, lexer, formatter).decode("utf-8")
    except:
        highlighted_text = pre(text)
    finally:
        return highlighted_text.replace("\n", "<br />\n")



from django import template

from mycms.models import CMSEntries

register = template.Library()


@register.tag
def PageNavigation(parser, token):

    return PageNavigationNode()


class PageNavigationNode(template.Node):
    def __init__(self):
        pass

    def render(self, context):
        # Load the template and return it
        self.request = template.Variable("request")
        t = template.loader.get_template("mycms/templatetags/PageNavigation.html")

        return t.render(context.flatten())


@register.tag
def MemberPageNavigation(parser, token):

    return MemberPageNavigationNode()


class MemberPageNavigationNode(template.Node):
    def __init__(self):

        pass

    def render(self, context):

        self.request = template.Variable("request")
        t = template.loader.get_template("mycms/templatetags/MemberPageNavigation.html")
        return t.render(context.flatten())


@register.inclusion_tag("mycms/templatetags/widgets/categories.html")
def categories_widget():
    """Get all the categories for the current page."""

    return {}


@register.inclusion_tag("mycms/templatetags/widgets/base_categories.html")
def base_categories_widget(cols):
    """Get all the categories for the current page."""

    def row_objects(obj_list):
        for i in range(0, len(obj_list), cols):
            yield obj_list[i : i + cols]

    obj_list = CMSEntries.objects.filter(
        path__parent__path="/", page_type__page_type="CATEGORY"
    )

    return {"categories": obj_list, "row_objects": row_objects(obj_list)}



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-06-26 14:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mycms", "0003_cmsentries_lists_include"),
    ]

    operations = [
        migrations.AddField(
            model_name="cmsentries",
            name="logo_url",
            field=models.CharField(
                blank=True,
                default="/static/mycms/images/png/default.png",
                max_length=1024,
                null=True,
            ),
        ),
    ]



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.utils.text import slugify

try:

    unicode("test")

except NameError as e:
    # we must be in python 3
    unicode = str


from loremipsum import generate_paragraphs
from mycms.creole import creole2html
from pprint import pformat
import simplejson as json

from django.conf import settings
import shlex


from xml.sax.saxutils import escape

try:
    from pygments import highlight
    from pygments.formatters.html import HtmlFormatter

    PYGMENTS = True
except ImportError:
    PYGMENTS = False

from mycms.creole.shared.utils import get_pygments_lexer, get_pygments_formatter


def html(text):
    """
    Macro tag <<html>>...<</html>>
    Pass-trought for html code (or other stuff)
    """
    return text


# ----------------------------------------------------------------------
def HTML(*args, **kwargs):
    """"""
    text = kwargs.get("text", None)
    return html(text)


def NEWLINE(*args, **kwargs):

    text = kwargs.get("text", 0)

    text_result = "</br>"
    try:
        numlines = int(text)
        for i in range(numlines):
            text_result += "</br>"
        return text_result

    except ValueError as e:
        return "[[ERROR: NEWLINE tag requires a number. {} was given".format(text)


def pre(text):
    """
    Macro tag <<pre>>...<</pre>>.
    Put text between html pre tag.
    """
    return "<pre>%s</pre>" % escape(text)


def code(*args, **kwargs):
    """
    Macro tag <<code ext=".some_extension">>...<</code>>
    If pygments is present, highlight the text according to the extension.
    """

    text = kwargs.get("text", None)
    ext = kwargs.get("ext", ".sh")
    nums = kwargs.get("nums", None)

    if not PYGMENTS:
        return pre(text)

    try:
        source_type = ""
        if "." in ext:
            source_type = ext.strip().split(".")[1]
        else:
            source_type = ext.strip()
    except IndexError:
        source_type = ""

    lexer = get_pygments_lexer(source_type, text)
    # formatter = get_pygments_formatter()

    try:
        if nums:
            formatter = HtmlFormatter(linenos="table", lineseparator="\n")
        else:
            formatter = HtmlFormatter(lineseparator="\n")

        # highlighted_text = highlight(text, lexer, formatter).decode('utf-8')
        # It seems with python3 there is no need to do a decode.
        highlighted_text = highlight(text, lexer, formatter)
    except Exception as e:
        print(e)
        highlighted_text = pre(text)
    # finally:
    #    return highlighted_text.replace('\n', '<br />\n')

    return highlighted_text


# ----------------------------------------------------------------------
def alertblock(*args, **kwargs):
    """"""
    text = kwargs.get("text", None)
    return template.format(text)


# ----------------------------------------------------------------------
def alertwarning(*args, **kwargs):
    """"""
    text = kwargs.get("text", None)
    template = """<div class="alert alert-warning">{}</div>"""
    return template.format(text)


# ----------------------------------------------------------------------
def alertsuccess(*args, **kwargs):
    """"""

    text = kwargs.get("text", None)
    template = """<div class="alert alert-success ">{}</div>"""
    return template.format(text)


# ----------------------------------------------------------------------
def alertinfo(*args, **kwargs):
    """"""
    text = kwargs.get("text", None)
    template = """<div class="alert alert-info">{}</div>"""
    return template.format(text)


# ----------------------------------------------------------------------
def H1(*args, **kwargs):
    """"""
    text = kwargs.get("text", None)
    template = """<a  name="{}"></a><h1 class="multipage-submenu-h1">{}</h2> """

    anchor_text_url = slugify(text)
    return template.format(text, anchor_text_url)


def H2(*args, **kwargs):
    """"""
    text = kwargs.get("text", None)
    template = """<h2 class="multipage-submenu-h2">{}</h2><a name="{}"></a> """
    anchor_text_url = slugify(text)

    return template.format(text, anchor_text_url)


# ----------------------------------------------------------------------
def infoblock(*args, **kwargs):
    """"""

    text = kwargs.get("text", "No text provided.")
    style = kwargs.get("style", "width: 400px; float: right; margin-left:10px")
    image = kwargs.get("image", None)
    author = kwargs.get("author", None)

    if image:
        image = (
            """<div class="quote-photo"><img src="img/temp/user.jpg" alt=""></div>"""
        )
    else:
        image = ""

    if author:
        author = """<div class="quote-author">James Livinston - <span>The New York Post</span></div>"""
    else:
        author = ""

    template = """
    <div class="boxinfo" style="{}">
        <div class="testimonials-user">{}<p>{}</p>{}</div>
</div>""".format(
        style, image, text, author
    )

    return template


# ----------------------------------------------------------------------
def image(*args, **kwargs):
    """
    We parse the content of the text to get the information about the image.
    """

    text = kwargs.get("text", None)
    name = kwargs.get("name", None)
    view = kwargs.get("view", None)
    class_ = kwargs.get("class", "article-image")
    style = kwargs.get("style", "width:80%")
    path_str = view.path_str

    img_url = "/static/assets/{}/{}".format(view.path_str, name)
    img = """<div class="image-holder">
    <img src="{}" class="{}" style="{}" />
    <p class="image-description">{}</p>
    </div>""".format(
        img_url, class_, style, text
    )

    return img


def imagelist(*args, **kwargs):

    text = kwargs.get("text", None)
    names_str = kwargs.get("names", None)
    class_ = kwargs.get("class", "article-image")
    style = kwargs.get("style", "width:80%")

    html = ""
    names_str = names_str.replace(",", " ")
    names = names_str.split(" ")
    view = kwargs.get("view", None)
    path_str = view.path_str

    if names != None:

        start = """
        <div>
        <div class="container" id="imagelist">
          <div class="row">
        """

        html = html + start

        for name in names:

            img_url = "/static/assets/{}/{}".format(view.path_str, name)
            content = """
                 <div class="col-sm image-holder">
                  <img src="{}" class="article-image clickable-image" style="width:80%" onclick="show_images_overlay(this)">
                </div>
            """
            html = html + content.format(img_url)

        end = """
            </div>
          </div>
        </div>   
        """

        html = html + end
    else:
        html = "Warning: No images were provided."

    return html


def google_addsense_code(*args, **kwargs):

    """"""
    text = kwargs.get("text", None)
    # template = """<h2 class="multipage-submenu-h2">{}</h2><a name="{}"></a> """
    # anchor_text_url = slugify(text)

    if settings.FORCE_SHOW_ADVERTS or (settings.DEBUG == False):

        code = """
<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
<!-- 200x200 -->
<ins class="adsbygoogle"
     style="display:inline-block;width:200px;height:200px"
     data-ad-client="ca-pub-9449210019187312"
     data-ad-slot="6494365200"></ins>
<script>
(adsbygoogle = window.adsbygoogle || []).push({});
</script>
	"""
    else:
        code = """<img src="/static/mycms/images/200x200.png">"""

    frame = """<div class="frame_200x200" style="float: right;width: 205px;height: 205px;padding-left:15px;">{}</div>""".format(
        code
    )

    return frame


# ----------------------------------------------------------------------
def debug(*args, **kwargs):

    """
    Just a simple example which shows the view's json_data.

    """

    view = kwargs.get("view", None)

    if view is None:
        return "MACRO: Debug did not get a view"

    result = "Object dictionary: {} ".format(view.json_data)

    return result


########################################################################
class CreoleFormatter(object):
    """"""

    # ----------------------------------------------------------------------
    def __init__(self, raw_content=None, view=None):
        """Constructor"""
        self.raw_content = raw_content
        self.view = view

    # ----------------------------------------------------------------------
    def html(self, fake_content=False, view=None):
        """Returns the html"""

        if view is None:
            view = self.view

        if fake_content:
            paragraphs = generate_paragraphs(5, start_with_lorem=False)
            p = ""
            for paragraph in paragraphs:
                p = unicode(paragraph[2]) + "\n\n" + p
            return creole2html(p)

        # The view object is actually passed to the macro being called such that
        # it can manipulate the view object to update it.
        return creole2html(
            self.raw_content,
            macros={
                "code": code,
                "pre": pre,
                "html": html,
                "HTML": HTML,
                "H1": H1,
                "H2": H2,
                "alertblock": alertblock,
                "alertsuccess": alertsuccess,
                "alertinfo": alertinfo,
                "alerterror": alertwarning,
                "infoblock": infoblock,
                "image": image,
                "debug": debug,
                "google_addsense_code": google_addsense_code,
                "imagelist": imagelist,
                "NEWLINE": NEWLINE,
            },
            verbose=None,
            stderr=None,
            view=view,
        )



from .mycms_view import ViewObject
from .category_page import CategoryPage
from .multi_page import MultiPage
from .single_page import SinglePage
from .multi_page import MemberPage
from .global_article_list import AllArticlesPage



"""
This module implements the various google add types that can 
be shown on MyCMS. 

It creates HTML for the adds where an example is as shown below:

    <script async src="https://pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
        <!-- DefaultHorizontal -->
        <ins class="adsbygoogle"
        style="display:block"
        data-ad-client="ca-pub-9449210019187312"
        data-ad-slot="1816544788"
        data-ad-format="auto"
        data-full-width-responsive="true"></ins>
    </script>

The above HTML is mostly the same for each type of add but some adds
have more parameters while some have less. 

The following two values are always present and never changes. This is 
specific to each mycms instance. This could be stored in the database 
or in the settings.py of mycms

        data-ad-client="ca-pub-9449210019187312"
        data-ad-slot="1816544788"

Since only a few things change, we have a base class and for each type of 
add we inheret from the base class and just add or override the required
parameters. 

The adds class then provides a dictionary to the template. 


"""


class BaseAdds(object):
    def __init__(self, dataadclient, dataadslot):
        self.dataadclient = dataadclient
        self.dataadslot = dataadslot

    def html(self):
        raise NotImplementedError("Implement in child class")

    def __str__(self):
        raise NotImplementedError("Implement this in chlid class")


class MyCMSGoogleAdds(BaseAdds):
    def __init__(self, dataadclient, dataadslot):
        pass

    def html(self):
        pass



#!/usr/bin/env python
# coding: utf-8

"""
    python-creole utils
    ~~~~~~~~~~~~~~~~~~~    


    :copyleft: 2008-2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import re

try:
    import htmlentitydefs as entities
except ImportError:
    from html import entities  # python 3

from mycms.creole.py3compat import PY3


entities_rules = "|".join(
    [
        r"(&\#(?P<number>\d+);)",
        r"(&\#x(?P<hex>[a-fA-F0-9]+);)",
        r"(&(?P<named>[a-zA-Z]+);)",
    ]
)
# print(entities_rules)
entities_regex = re.compile(entities_rules, re.VERBOSE | re.UNICODE | re.MULTILINE)


class Deentity(object):
    """
    replace html entity

    >>> d = Deentity()
    >>> d.replace_all("-=[&nbsp;&gt;&#62;&#x3E;nice&lt;&#60;&#x3C;&nbsp;]=-")
    '-=[ >>>nice<<< ]=-'
        
    >>> d.replace_all("-=[M&uuml;hlheim]=-") # uuml - latin small letter u with diaeresis
    '-=[M\\xfchlheim]=-'

    >>> d.replace_number("126")
    '~'
    >>> d.replace_hex("7E")
    '~'
    >>> d.replace_named("amp")
    '&'
    """

    def replace_number(self, text):
        """ unicode number entity """
        unicode_no = int(text)
        if PY3:
            return chr(unicode_no)
        else:
            return unichr(unicode_no)

    def replace_hex(self, text):
        """ hex entity """
        unicode_no = int(text, 16)
        if PY3:
            return chr(unicode_no)
        else:
            return unichr(unicode_no)

    def replace_named(self, text):
        """ named entity """
        if text == "nbsp":
            # Non breaking spaces is not in htmlentitydefs
            return " "
        else:
            codepoint = entities.name2codepoint[text]
            if PY3:
                return chr(codepoint)
            else:
                return unichr(codepoint)

    def replace_all(self, content):
        """ replace all html entities form the given text. """

        def replace_entity(match):
            groups = match.groupdict()
            for name, text in groups.items():
                if text is not None:
                    replace_method = getattr(self, "replace_%s" % name)
                    return replace_method(text)

            # Should never happen:
            raise RuntimeError("deentitfy re rules wrong!")

        return entities_regex.sub(replace_entity, content)


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())



# coding: utf-8


"""
    python-creole
    ~~~~~~~~~~~~~

    :homepage:
      http://code.google.com/p/python-creole/
    
    :sourcecode:
      http://github.com/jedie/python-creole
    
    :PyPi:
      http://pypi.python.org/pypi/python-creole/

    :copyleft: 2008-2014 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

__version__ = (1, 2, 1)
__api__ = (1, 0)  # Creole 1.0 spec - http://wikicreole.org/


import os
import sys
import warnings

from mycms.creole.creole2html.emitter import HtmlEmitter
from mycms.creole.creole2html.parser import BlockRules, CreoleParser
from mycms.creole.html_parser.parser import HtmlParser
from mycms.creole.py3compat import TEXT_TYPE


# TODO: Add git date to __version__


VERSION_STRING = ".".join(str(part) for part in __version__)
API_STRING = ".".join(str(integer) for integer in __api__)


def creole2html(
    markup_string,
    debug=False,
    parser_kwargs={},
    emitter_kwargs={},
    block_rules=None,
    blog_line_breaks=True,
    macros=None,
    verbose=None,
    stderr=None,
    view=None,
):
    """
    convert creole markup into html code

    >>> creole2html('This is **creole //markup//**!')
    '<p>This is <strong>creole <i>markup</i></strong>!</p>'
    
    Info: parser_kwargs and emitter_kwargs are deprecated
    """
    assert isinstance(markup_string, TEXT_TYPE), "given markup_string must be unicode!"

    parser_kwargs2 = {
        "block_rules": block_rules,
        "blog_line_breaks": blog_line_breaks,
    }
    if parser_kwargs:
        warnings.warn(
            "parser_kwargs argument in creole2html would be removed in the future!",
            PendingDeprecationWarning,
        )
        parser_kwargs2.update(parser_kwargs)

    # Create document tree from mycms.creole markup
    document = CreoleParser(markup_string, **parser_kwargs2).parse()
    if debug:
        document.debug()

    emitter_kwargs2 = {
        "macros": macros,
        "verbose": verbose,
        "stderr": stderr,
        "view": view,
    }
    if emitter_kwargs:
        warnings.warn(
            "emitter_kwargs argument in creole2html would be removed in the future!",
            PendingDeprecationWarning,
        )
        emitter_kwargs2.update(emitter_kwargs)

    # Build html code from document tree
    return HtmlEmitter(document, **emitter_kwargs2).emit()


def parse_html(html_string, debug=False):
    """ create the document tree from html code """
    assert isinstance(html_string, TEXT_TYPE), "given html_string must be unicode!"

    h2c = HtmlParser(debug=debug)
    document_tree = h2c.feed(html_string)
    if debug:
        h2c.debug()
    return document_tree


if __name__ == "__main__":
    print("runing local doctest...")
    import doctest

    print(
        doctest.testmod(
            #            verbose=True
        )
    )



# encoding: utf-8
from django.http import HttpResponse
import json

MIMEANY = "*/*"
MIMEJSON = "application/json"
MIMETEXT = "text/plain"


def response_mimetype(request):
    """response_mimetype -- Return a proper response mimetype, accordingly to
    what the client accepts, as available in the `HTTP_ACCEPT` header.
    request -- a HttpRequest instance.
    """
    can_json = MIMEJSON in request.META["HTTP_ACCEPT"]
    can_json |= MIMEANY in request.META["HTTP_ACCEPT"]
    return MIMEJSON if can_json else MIMETEXT


class JSONResponse(HttpResponse):
    """JSONResponse -- Extends HTTPResponse to handle JSON format response.
    This response can be used in any view that should return a json stream of
    data.
    Usage:
        def a_iew(request):
            content = {'key': 'value'}
            return JSONResponse(content, mimetype=response_mimetype(request))
    """

    def __init__(self, obj="", json_opts=None, mimetype=MIMEJSON, *args, **kwargs):
        json_opts = json_opts if isinstance(json_opts, dict) else {}
        content = json.dumps(obj, **json_opts)
        super(JSONResponse, self).__init__(content, mimetype, *args, **kwargs)



from faker import Faker
import random


def get_fake_contents(num_paragraphs):
    fake = Faker()

    paragraphs = []
    for c in range(num_paragraphs):
        num_sentences = random.randint(5, 10)
        paragraphs.append(" ".join(fake.paragraphs(num_sentences)))

    return "\n\n".join(paragraphs)



from mycms.models import CMSEntries
from mycms.models import CMSPageTypes
from mycms.view_handlers.mycms_view import ViewObject

from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.db.utils import OperationalError

import logging

logger = logging.getLogger("mycms.page_handlers")


class BasePage(object):
    def get_list_params(self):
        """
        Used by pages that recieves a limit and a page number. 
        This is here to simply getting the parameters from the 
        request. 
        """

        LIMIT = 10
        if hasattr(self, "request"):
            try:
                limit = int(self.request.GET.get("limit", LIMIT))
            except Exception as e:
                # this can only happen if we got a bad limit value such as a non
                # integer value.Set limit to a sane value.
                limit = LIMIT

            try:
                page = int(self.request.GET.get("page", 1))
            except Exception as e:
                page = 1

            try:
                offset = int(self.request.GET.get("offset"))
            except Exception as e:
                # same reason as above
                offset = (page - 1) * limit

        else:
            limit = LIMIT
            offset = 0
            page = 1

        return limit, offset, page


try:

    try:
        singlepageview_pagetype_obj, c = obj = CMSPageTypes.objects.get_or_create(
            page_type="SINGLEPAGE",
            text="Single Page HTML",
            view_class="SinglePage",
            view_template="SinglePage.html",
        )

    except ObjectDoesNotExist as e:
        singlepageview_pagetype_obj = CMSPageTypes(
            page_type="SINGLEPAGE",
            text="Single Page HTML",
            view_class="SinglePage",
            view_template="SinglePage.html",
        )
        singlepageview_pagetype_obj.save()

    except MultipleObjectsReturned as e:
        msg = "Got more than 1 CMSPageTypes : SINGLEPAGE. Database is inconsistent, Will return the first one. "
        logger.warn(msg)

        singlepageview_pagetype_obj = CMSPageTypes.objects.filter(
            page_type="SINGLEPAGE"
        )[0]

    try:
        categorypageview_pagetype_obj = CMSPageTypes.objects.get(page_type="CATEGORY")

    except ObjectDoesNotExist as e:

        msg = "Could not load CATEGORY view object. Going to create it."
        logger.debug(msg)
        pagetype_obj, _ = CMSPageTypes.objects.get_or_create(
            page_type="CATEGORY",
            text="Category Page",
            view_class="CategoryPage",
            view_template="CategoryPage.html",
        )

    except MultipleObjectsReturned as e:
        msg = "Got more than 1 CMSPageType: CATEGORY. Database is inconsistent. Will return the first one."
        logger.info(msg)

        categorypageview_pagetype_obj = CMSPageTypes.objects.filter(
            page_type="CATEGORY"
        )[0]

    try:
        multipageview_pagetype_obj = CMSPageTypes.objects.get(page_type="MULTIPAGE")

    except ObjectDoesNotExist as e:

        msg = "Could not load MULTIPAGE view object. Going to create it."
        logger.debug(msg)
        multipageview_pagetype_obj, _ = CMSPageTypes.objects.get_or_create(
            page_type="MULTIPAGE",
            text="MultPage Article",
            view_class="MultiPage",
            view_template="MultiPage.html",
        )

    except MultipleObjectsReturned as e:
        msg = "Got more than 1 CMSMultiPageType: MULTIPAGE. Database is inconsistent. Will return the first one."
        logger.info(msg)

        multipageview_pagetype_obj = CMSMultiPageTypes.objects.filter(
            page_type="MULTIPAGE"
        )[0]

    # MULTIPAGE

    try:
        memberpageview_pagetype_obj = CMSPageTypes.objects.get(page_type="MEMBERPAGE")

    except ObjectDoesNotExist as e:

        msg = "Could not load MULTIPAGE view object. Going to create it."
        logger.debug(msg)
        pagetype_obj, _ = CMSPageTypes.objects.get_or_create(
            page_type="MEMBERPAGE",
            text="MemberPage Article",
            view_class="MemberPage",
            view_template="MemberPage.html",
        )

    except MultipleObjectsReturned as e:
        msg = "Got more than 1 CMSMultiPageType: MULTIPAGE. Database is inconsistent. Will return the first one."
        logger.info(msg)

        memberpageview_pagetype_obj = CMSMultiPageTypes.objects.filter(
            page_type="MEMBERPAGE"
        )[0]

    try:
        allarticles_pagetype_obj, c = obj = CMSPageTypes.objects.get_or_create(
            page_type="ALLARTICLES",
            text="All Articles ",
            view_class="AllArticlesPage",
            view_template="AllArticlesPage.html",
        )

    except ObjectDoesNotExist as e:
        allarticles_pagetype_obj = CMSPageTypes(
            ppage_type="ALLARTICLES",
            text="All Articles ",
            view_class="AllArticlesPage",
            view_template="AllArticlesPage.html",
        )
        allarticles_pagetype_obj.save()

    except MultipleObjectsReturned as e:
        msg = "Got more than 1 CMSPageTypes : SINGLEPAGE. Database is inconsistent, Will return the first one. "
        logger.warn(msg)

        allarticles_pagetype_obj = CMSPageTypes.objects.filter(page_type="ALLARTICLES")[
            0
        ]


except OperationalError as e:
    # This can happen only when the database is not yet initialized.
    logger.warn(
        "Failed to create page types because database is probably not yet initialized."
    )



#!/usr/bin/env python
# coding: utf-8

"""
    python-creole commandline interface
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2013 by the python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals
import argparse
import codecs

from mycms.creole import creole2html, html2creole, html2rest, html2textile
from mycms.creole import VERSION_STRING


class CreoleCLI(object):
    def __init__(self, convert_func):
        self.convert_func = convert_func
        self.parser = argparse.ArgumentParser(
            description=(
                "python-creole is an open-source (GPL) markup converter"
                " in pure Python for:"
                " creole2html, html2creole, html2ReSt, html2textile"
            ),
            version=VERSION_STRING,
        )

        self.parser.add_argument("sourcefile", help="source file to convert")
        self.parser.add_argument("destination", help="Output filename")
        self.parser.add_argument(
            "--encoding",
            default="utf-8",
            help="Codec for read/write file (default encoding: utf-8)",
        )

        args = self.parser.parse_args()

        sourcefile = args.sourcefile
        destination = args.destination
        encoding = args.encoding

        self.convert(sourcefile, destination, encoding)

    def convert(self, sourcefile, destination, encoding):
        print(
            "Convert %r to %r with %s (codec: %s)"
            % (sourcefile, destination, self.convert_func.__name__, encoding)
        )

        with codecs.open(sourcefile, "r", encoding=encoding) as infile:
            with codecs.open(destination, "w", encoding=encoding) as outfile:
                content = infile.read()
                converted = self.convert_func(content)
                outfile.write(converted)
        print("done. %r created." % destination)


def cli_creole2html():
    cli = CreoleCLI(creole2html)


#     cli.convert()


def cli_html2creole():
    cli = CreoleCLI(html2creole)


#     cli.convert()


def cli_html2rest():
    cli = CreoleCLI(html2rest)


#     cli.convert()


def cli_html2textile():
    cli = CreoleCLI(html2textile)


#     cli.convert()


if __name__ == "__main__":
    import sys

    sys.argv += ["../README.creole", "../test.html"]
    print(sys.argv)
    cli_creole2html()



from django import template
from django.core.exceptions import ObjectDoesNotExist
from django.template import Template, Context
from django.template.loader import get_template
from django.contrib.auth.models import User
from django.core.paginator import Paginator


from django.template.loader import render_to_string

# from mycms.models import Paths, Pages
# from mycms.pageview.base import get_pageview

from mycms.models import CMSPaths
from mycms.models import CMSEntries

register = template.Library()


@register.inclusion_tag("mycms/templatetags/cmsentries/frontpage.html")
def frontpage():
    """
    The {%frontpage%} tag inserts a listing of all cmsentries that are marked 
    published and frontpaged.
    
    """
    cmsentries = CMSEntries.objects.filter(frontpage=True, published=True).order_by(
        "-date_created"
    )[:10]
    paginator = Paginator(cmsentries, 4)

    return {"cmsentries": cmsentries}


@register.inclusion_tag("mycms/templatetags/archives.html")
def archives():
    return None



#!/usr/bin/env python
# coding: utf-8

"""
    Try to be so strict as PyPi.

    Code based on:
    https://bitbucket.org/pypa/pypi/src/tip/description_utils.py

    see also:
    https://bitbucket.org/pypa/pypi/issue/161/rest-formatting-fails-and-there-is-no-way
"""

from __future__ import division, absolute_import, print_function

try:
    # Python 3
    from urllib.parse import urlparse
except ImportError:
    # Python 2
    from urlparse import urlparse

from mycms.creole.exceptions import DocutilsImportError

try:
    import docutils
    from docutils import io, readers
    from docutils.core import publish_doctree, Publisher
    from docutils.writers import get_writer_class
    from docutils.transforms import TransformError, Transform
except ImportError as err:
    msg = (
        "%s - You can't use rest2html!"
        " Please install: http://pypi.python.org/pypi/docutils"
    ) % err
    raise DocutilsImportError(msg)


ALLOWED_SCHEMES = """file ftp gopher hdl http https imap mailto mms news nntp
prospero rsync rtsp rtspu sftp shttp sip sips snews svn svn+ssh telnet
wais irc""".split()


def pypi_rest2html(source, output_encoding="unicode"):
    """
    >>> pypi_rest2html("test!")
    u'<p>test!</p>\n'
    """
    settings_overrides = {
        "raw_enabled": 0,  # no raw HTML code
        "file_insertion_enabled": 0,  # no file/URL access
        "halt_level": 2,  # at warnings or errors, raise an exception
        "report_level": 5,  # never report problems with the reST code
    }

    # Convert reStructuredText to HTML using Docutils.
    document = publish_doctree(source=source, settings_overrides=settings_overrides)

    for node in document.traverse():
        if node.tagname == "#text":
            continue
        if node.hasattr("refuri"):
            uri = node["refuri"]
        elif node.hasattr("uri"):
            uri = node["uri"]
        else:
            continue
        o = urlparse(uri)
        if o.scheme not in ALLOWED_SCHEMES:
            raise TransformError("link scheme not allowed")

    # now turn the transformed document into HTML
    reader = readers.doctree.Reader(parser_name="null")
    pub = Publisher(
        reader, source=io.DocTreeInput(document), destination_class=io.StringOutput
    )
    pub.set_writer("html")
    pub.process_programmatic_settings(None, settings_overrides, None)
    pub.set_destination(None, None)
    pub.publish()
    parts = pub.writer.parts

    output = parts["body"]

    if output_encoding != "unicode":
        output = output.encode(output_encoding)

    return output


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())



from django.conf.urls import include, url
from django.views.decorators.csrf import csrf_exempt
from mycms.views import (
    CMSContentsAPIView,
    CMSEntriesAPIView,
    CMSMarkUpsAPIView,
    CMSTemplatesAPIView,
    CMSPageView,
    CMSPathsAPIView,
    CMSEntriesROAPIView,
    LoremIpsumAPIView,
    AssetsUploaderView,
    CMSPageTypesAPIView,
    CMSFileUpload,
)

from rest_framework.documentation import include_docs_urls
from rest_framework.schemas import get_schema_view
from rest_framework.authtoken import views as authtoken_views

from mycms.views import MockupLoader
from mycms.views import CMSUserContentArea
from mycms.views import CMSLoginView
from mycms.views import CMSLogoutView
from mycms.views import CMSUserProfileView

from rest_framework import routers

from mycms import api

from mycms.views import CMSFormatterContent

urlpatterns = [
    url(r"^search/", include("haystack.urls")),
    url(r"^logout/$", CMSLogoutView.as_view()),
    url(r"^profile/$", CMSUserProfileView.as_view()),
    url(r"^login/$", CMSLoginView.as_view()),
    url(
        r"^(?P<path>[-/\.a-z\d_]*)/assets_manager/$",
        csrf_exempt(AssetsUploaderView.as_view()),
        name="assets_manager_get",
    ),
    url(
        r"^(?P<path>[-/\.a-z\d_]*)/assets_manager/(?P<filename>[-/\.a-z\d_A-Z]*)$",
        csrf_exempt(AssetsUploaderView.as_view()),
        name="assets_manager_get",
    ),
    url(
        r"assets_manager/$",
        csrf_exempt(AssetsUploaderView.as_view()),
        name="assets_manager_get",
    ),
    url(
        r"assets_manager/(?P<filename>[-/\.a-z\d_A-Z]*)$",
        csrf_exempt(AssetsUploaderView.as_view()),
        name="assets_manager_get",
    ),
    url(r"^mockups/(?P<template>[-._\w\W\d]*.html)$", MockupLoader.as_view()),
    url(r"^mockups/?$", MockupLoader.as_view()),
    url(r"^user/admin/articles/?$", CMSUserContentArea.as_view()),
    url(r"^(?P<path>[-/\.a-z\d_]*)/$", CMSPageView.as_view(), name="cms_page"),
    url(r"^$", CMSPageView.as_view(), name="cms_page"),
]


schema_view = get_schema_view(title="MyCMS API")

cms_root = [
    url(r"^$", CMSPageView.as_view(), name="cms_page"),
    url(r"^api/v2/docs/", include_docs_urls(title="MyCMS API")),
    url("^api/v2/schemas/", schema_view),
    url(
        r"api/v2/cmsauthtoken",
        api.CMSAuthToken.as_view({"post": "retrieve"}),
        name="cmsauthtoken",
    ),
    url(
        r"api/v2/cmspreview",
        api.CMSContentPreview.as_view({"post": "retrieve"}),
        name="cmspreview",
    ),
]


router = routers.DefaultRouter()
# router.register(r'zones/(?P<zone_name>[-/\.a-z\d_]*)/records', RecordsViewSet, basename='Records')
router.register(r"api/v2/cmscontents", api.CMSContentsViewSet, basename="cmscontents")
router.register(r"api/v2/cmsentries", api.CMSEntriesViewSet, basename="cmsentries")
router.register(r"api/v2/cmspaths", api.CMSPathsViewSet, basename="cmspaths")
router.register(r"api/v2/cmspages", api.CMSPagesViewSet, basename="cmspages")
router.register(
    r"api/v2/cmspagetypes", api.CMSPageTypeViewSet, basename="cmspagetypes"
)
# router.register(r'api/v2/cmspreview', api.CMSContentPreview, basename='cmspreview')
# router.register(r'api/v2/cmsauthtoken', api.CMSAuthToken, basename='cmsauthtoken')
# router.register(r'api/v2/utils/cmsformatter/(?P<content_id>[\d]*)/$', CMSFormatterContent, basename='cmsformatter')

urlpatterns = cms_root + router.urls + urlpatterns

# for i in urlpatterns:
#    print(i)



from django import template

register = template.Library()


@register.inclusion_tag("mycms/templatetags/category_editor.html", takes_context=True)
def category_editor(context, *args, **kwargs):

    script_list = context.get("script_list", [])
    return {"script_list": script_list}



#!/usr/bin/env python
# coding: utf-8

"""
    Base document tree emitter
    ~~~~~~~~~~~~~~~~~~~~~~~~~~


    :copyleft: 2008-2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals
import posixpath

from mycms.creole.html_parser.config import BLOCK_TAGS
from mycms.creole.html_tools.deentity import Deentity
from mycms.creole.py3compat import TEXT_TYPE
from mycms.creole.shared.markup_table import MarkupTable
from mycms.creole.shared.unknown_tags import transparent_unknown_nodes


class BaseEmitter(object):
    """
    Build from a document_tree (html2creole.parser.HtmlParser instance) a
    creole markup text.
    """

    def __init__(self, document_tree, unknown_emit=None, debug=False):
        self.root = document_tree

        if unknown_emit is None:
            self._unknown_emit = transparent_unknown_nodes
        else:
            self._unknown_emit = unknown_emit

        self.last = None
        self.debugging = debug

        self.deentity = Deentity()  # for replacing html entities
        self._inner_list = ""
        self._mask_linebreak = False

    # --------------------------------------------------------------------------

    def blockdata_pass_emit(self, node):
        return "%s\n\n" % node.content
        return node.content

    # --------------------------------------------------------------------------

    def data_emit(self, node):
        # node.debug()
        return node.content

    def entityref_emit(self, node):
        """
        emit a named html entity
        """
        entity = node.content

        try:
            return self.deentity.replace_named(entity)
        except KeyError as err:
            if self.debugging:
                print("unknown html entity found: %r" % entity)
            return "&%s" % entity  # FIXME
        except UnicodeDecodeError as err:
            raise UnicodeError("Error handling entity %r: %s" % (entity, err))

    def charref_emit(self, node):
        """
        emit a not named html entity
        """
        entity = node.content

        if entity.startswith("x"):
            # entity in hex
            hex_no = entity[1:]
            return self.deentity.replace_hex(hex_no)
        else:
            # entity as a unicode number
            return self.deentity.replace_number(entity)

    # --------------------------------------------------------------------------

    def p_emit(self, node):
        return "%s\n\n" % self.emit_children(node)

    def br_emit(self, node):
        if self._inner_list != "":
            return "\\\\"
        else:
            return "\n"

    # --------------------------------------------------------------------------

    def _typeface(self, node, key):
        return key + self.emit_children(node) + key

    # --------------------------------------------------------------------------

    def li_emit(self, node):
        content = self.emit_children(node)
        return "\n%s %s" % (self._inner_list, content)

    def _list_emit(self, node, list_type):
        start_newline = False
        if self.last and self.last.kind not in BLOCK_TAGS:
            if not self.last.content or not self.last.content.endswith("\n"):
                start_newline = True

        if self._inner_list == "":  # Start a new list
            self._inner_list = list_type
        else:
            self._inner_list += list_type

        content = "%s" % self.emit_children(node)

        self._inner_list = self._inner_list[:-1]

        if self._inner_list == "":  # Start a new list
            if start_newline:
                return "\n" + content + "\n\n"
            else:
                return content.strip() + "\n\n"
        else:
            return content

    # --------------------------------------------------------------------------

    def table_emit(self, node):
        self._table = MarkupTable(
            head_prefix=self.table_head_prefix,
            auto_width=self.table_auto_width,
            debug_msg=self.debug_msg,
        )
        self.emit_children(node)
        content = self._table.get_table_markup()
        return "%s\n" % content

    def tr_emit(self, node):
        self._table.add_tr()
        self.emit_children(node)
        return ""

    def _escape_linebreaks(self, text):
        text = text.strip()
        text = text.split("\n")
        lines = [line.strip() for line in text]
        lines = [line for line in lines if line]
        content = "\\\\".join(lines)
        content = content.strip("\\")
        return content

    def th_emit(self, node):
        content = self.emit_children(node)
        content = self._escape_linebreaks(content)
        self._table.add_th(content)
        return ""

    def td_emit(self, node):
        content = self.emit_children(node)
        content = self._escape_linebreaks(content)
        self._table.add_td(content)
        return ""

    # --------------------------------------------------------------------------

    def _emit_content(self, node):
        content = self.emit_children(node)
        content = self._escape_linebreaks(content)
        if node.kind in BLOCK_TAGS:
            content = "%s\n\n" % content
        return content

    def div_emit(self, node):
        return self._emit_content(node)

    def span_emit(self, node):
        return self._emit_content(node)

    # --------------------------------------------------------------------------

    def document_emit(self, node):
        self.last = node
        return self.emit_children(node)

    def emit_children(self, node):
        """Emit all the children of a node."""
        return "".join(self.emit_children_list(node))

    def emit_children_list(self, node):
        """Emit all the children of a node."""
        self.last = node
        result = []
        for child in node.children:
            content = self.emit_node(child)
            assert isinstance(content, TEXT_TYPE)
            result.append(content)
        return result

    def emit_node(self, node):
        """Emit a single node."""

        def unicode_error(method_name, method, node, content):
            node.debug()
            raise AssertionError(
                "Method '%s' (%s) returns no unicode - returns: %s (%s)"
                % (method_name, method, repr(content), type(content))
            )

        if node.level:
            self.debug_msg(
                "emit_node",
                "%s (level: %i): %r" % (node.kind, node.level, node.content),
            )
        else:
            self.debug_msg("emit_node", "%s: %r" % (node.kind, node.content))

        method_name = "%s_emit" % node.kind
        emit_method = getattr(self, method_name, None)

        if emit_method:
            content = emit_method(node)
            if not isinstance(content, TEXT_TYPE):
                unicode_error(method_name, emit_method, node, content)
        else:
            content = self._unknown_emit(self, node)
            if not isinstance(content, TEXT_TYPE):
                unicode_error(method_name, self._unknown_emit, node, content)

        self.last = node
        return content

    #    def emit(self):
    #        """Emit the document represented by self.root DOM tree."""
    #        result = self.emit_node(self.root)
    ##        return result.strip() # FIXME
    #        return result.rstrip() # FIXME

    # -------------------------------------------------------------------------

    def debug_msg(self, method, txt):
        if not self.debugging:
            return
        print("%13s: %s" % (method, txt))
